@@ -13,17 +13,16 @@ import threading
 import time
 from copy import deepcopy
 from multiprocessing import Pool
-from multiprocessing.pool import ApplyResult
 from pathlib import Path, PosixPath
 from shutil import which
-from typing import Dict, Type
+from typing import Optional
 
 import psutil
 import yaml
 
 from ._citation import _write_citation_files
-from ._config import DIAGNOSTICS, TAGS
 from ._provenance import TrackedFile, get_task_provenance
+from .config._diagnostics import DIAGNOSTICS, TAGS
 
 
 def path_representer(dumper, data):
@@ -145,7 +144,7 @@ def _py2ncl(value, var_name=''):
     txt = var_name + ' = ' if var_name else ''
     if value is None:
         txt += '_Missing'
-    elif isinstance(value, str):
+    elif isinstance(value, (str, Path)):
         txt += '"{}"'.format(value)
     elif isinstance(value, (list, tuple)):
         if not value:
@@ -275,7 +274,7 @@ class BaseTask:
         """Return a mapping of product attributes."""
         return {
             product.filename: product.attributes
-            for product in self.products
+            for product in sorted(self.products)
         }
 
     def print_ancestors(self):
@@ -632,7 +631,7 @@ class DiagnosticTask(BaseTask):
                 attrs[key] = self.settings[key]
 
         ancestor_products = {
-            p.filename: p
+            str(p.filename): p
             for a in self.ancestors for p in a.products
         }
 
@@ -711,7 +710,7 @@ class TaskSet(set):
                 independent_tasks.add(task)
         return independent_tasks
 
-    def run(self, max_parallel_tasks: int = None) -> None:
+    def run(self, max_parallel_tasks: Optional[int] = None) -> None:
         """Run tasks.
 
         Parameters
@@ -736,7 +735,7 @@ class TaskSet(set):
     def _run_parallel(self, max_parallel_tasks=None):
         """Run tasks in parallel."""
         scheduled = self.flatten()
-        running: Dict[Type[BaseTask], Type[ApplyResult]] = {}
+        running = {}
 
         n_tasks = n_scheduled = len(scheduled)
         n_running = 0
