@@ -1,21 +1,19 @@
-"""Fixes for ACCESS1-3 model."""
+"""Fixes for the ACCESS1-0 model."""
 import iris
 from cf_units import Unit
 
+from ..common import ClFixHybridHeightCoord
 from ..fix import Fix
-from .access1_0 import Cl as BaseCl
-
-
-Cl = BaseCl
 
 
 class AllVars(Fix):
     """Common fixes to all vars."""
 
     def fix_metadata(self, cubes):
-        """Fix metadata.
+        """
+        Fix metadata.
 
-        Fixes wrong calendar 'gregorian' instead of 'proleptic_gregorian'
+        Fixes wrong calendar 'gregorian' instead of 'proleptic_gregorian'.
 
         Parameters
         ----------
@@ -38,3 +36,28 @@ class AllVars(Fix):
                                             calendar='proleptic_gregorian'))
                     time.units = Unit(time.units.name, 'gregorian')
         return cubes
+
+
+class Cl(ClFixHybridHeightCoord):
+    """Fixes for ``cl``."""
+
+    def fix_metadata(self, cubes):
+        """Remove attributes from ``vertical coordinate formula term: b(k)``.
+
+        Additionally add pressure level coordiante.
+
+        Parameters
+        ----------
+        cubes : iris.cube.CubeList
+            Input cubes which need to be fixed.
+
+        Returns
+        -------
+        iris.cube.CubeList
+
+        """
+        cubes = super().fix_metadata(cubes)
+        cube = self.get_cube_from_list(cubes)
+        coord = cube.coord(long_name='vertical coordinate formula term: b(k)')
+        coord.attributes = {}
+        return cubes
