@@ -5,14 +5,18 @@ import logging
 import iris
 import numpy as np
 
+from esmvalcore.iris_helpers import date2num
+
 from ..fix import Fix
 from ..shared import add_scalar_height_coord
+from ...table import CMOR_TABLES
 
 logger = logging.getLogger(__name__)
 
 
 def get_frequency(cube):
     """Determine time frequency of input cube."""
+
     try:
         time = cube.coord(axis='T')
     except iris.exceptions.CoordinateNotFoundError:
@@ -72,17 +76,33 @@ def divide_by_gravity(cube):
 
 class Clt(Fix):
     """Fixes for clt."""
+
     def fix_metadata(self, cubes):
+        """Fix metadata."""
         for cube in cubes:
             # Invalid input cube units (ignored on load) were '0-1'
             cube.units = '%'
-            cube.data = cube.core_data()*100.
+            cube.data = cube.core_data() * 100.
+
+        return cubes
+
+
+class Cl(Fix):
+    """Fixes for cl."""
+
+    def fix_metadata(self, cubes):
+        """Fix metadata."""
+        for cube in cubes:
+            # Invalid input cube units (ignored on load) were '0-1'
+            cube.units = '%'
+            cube.data = cube.core_data() * 100.
 
         return cubes
 
 
 class Evspsbl(Fix):
     """Fixes for evspsbl."""
+
     def fix_metadata(self, cubes):
         """Fix metadata."""
         for cube in cubes:
@@ -97,6 +117,7 @@ class Evspsbl(Fix):
 
 class Evspsblpot(Fix):
     """Fixes for evspsblpot."""
+
     def fix_metadata(self, cubes):
         """Fix metadata."""
         for cube in cubes:
@@ -111,6 +132,7 @@ class Evspsblpot(Fix):
 
 class Mrro(Fix):
     """Fixes for mrro."""
+
     def fix_metadata(self, cubes):
         """Fix metadata."""
         for cube in cubes:
@@ -123,6 +145,7 @@ class Mrro(Fix):
 
 class Orog(Fix):
     """Fixes for orography."""
+
     def fix_metadata(self, cubes):
         """Fix metadata."""
         fixed_cubes = []
@@ -135,6 +158,7 @@ class Orog(Fix):
 
 class Pr(Fix):
     """Fixes for pr."""
+
     def fix_metadata(self, cubes):
         """Fix metadata."""
         for cube in cubes:
@@ -147,6 +171,7 @@ class Pr(Fix):
 
 class Prsn(Fix):
     """Fixes for prsn."""
+
     def fix_metadata(self, cubes):
         """Fix metadata."""
         for cube in cubes:
@@ -161,6 +186,7 @@ class Prsn(Fix):
 
 class Ptype(Fix):
     """Fixes for ptype."""
+
     def fix_metadata(self, cubes):
         """Fix metadata."""
         for cube in cubes:
@@ -171,6 +197,7 @@ class Ptype(Fix):
 
 class Rlds(Fix):
     """Fixes for Rlds."""
+
     def fix_metadata(self, cubes):
         """Fix metadata."""
         for cube in cubes:
@@ -209,6 +236,7 @@ class Rlus(Fix):
 
 class Rls(Fix):
     """Fixes for Rls."""
+
     def fix_metadata(self, cubes):
         """Fix metadata."""
         for cube in cubes:
@@ -220,6 +248,7 @@ class Rls(Fix):
 
 class Rsds(Fix):
     """Fixes for Rsds."""
+
     def fix_metadata(self, cubes):
         """Fix metadata."""
         for cube in cubes:
@@ -258,6 +287,7 @@ class Rsus(Fix):
 
 class Rsdt(Fix):
     """Fixes for Rsdt."""
+
     def fix_metadata(self, cubes):
         """Fix metadata."""
         for cube in cubes:
@@ -270,6 +300,7 @@ class Rsdt(Fix):
 
 class Rss(Fix):
     """Fixes for Rss."""
+
     def fix_metadata(self, cubes):
         """Fix metadata."""
         for cube in cubes:
@@ -282,6 +313,7 @@ class Rss(Fix):
 
 class Tasmax(Fix):
     """Fixes for tasmax."""
+
     def fix_metadata(self, cubes):
         for cube in cubes:
             fix_hourly_time_coordinate(cube)
@@ -290,6 +322,7 @@ class Tasmax(Fix):
 
 class Tasmin(Fix):
     """Fixes for tasmin."""
+
     def fix_metadata(self, cubes):
         for cube in cubes:
             fix_hourly_time_coordinate(cube)
@@ -298,6 +331,7 @@ class Tasmin(Fix):
 
 class Zg(Fix):
     """Fixes for Geopotential."""
+
     def fix_metadata(self, cubes):
         """Fix metadata."""
         for cube in cubes:
@@ -307,6 +341,7 @@ class Zg(Fix):
 
 class AllVars(Fix):
     """Fixes for all variables."""
+
     def _fix_coordinates(self, cube):
         """Fix coordinates."""
         # Fix coordinate increasing direction
@@ -326,6 +361,13 @@ class AllVars(Fix):
 
         for coord_def in self.vardef.coordinates.values():
             axis = coord_def.axis
+            # ERA5 uses regular pressure level coordinate. In case the cmor
+            # variable requires a hybrid level coordinate, we replace this with
+            # a regular pressure level coordinate.
+            # (https://github.com/ESMValGroup/ESMValCore/issues/1029)
+            if axis == "" and coord_def.name == "alevel":
+                axis = "Z"
+                coord_def = CMOR_TABLES['CMIP6'].coords['plev19']
             coord = cube.coord(axis=axis)
             if axis == 'T':
                 coord.convert_units('days since 1850-1-1 00:00:00.0')
@@ -356,7 +398,7 @@ class AllVars(Fix):
                     month = 1
                     year = year + 1
                 end.append(cell.point.replace(month=month, year=year))
-            end = coord.units.date2num(end)
+            end = date2num(end, coord.units)
             start = coord.points
             coord.points = 0.5 * (start + end)
             coord.bounds = np.column_stack([start, end])
