@@ -1,12 +1,11 @@
 """API for recipe metadata."""
 
-from pathlib import Path
+from typing import Optional
 
 import pybtex
 from pybtex.database.input import bibtex
 
-from esmvalcore._citation import REFERENCES_PATH
-from esmvalcore._config import TAGS
+from esmvalcore.config._diagnostics import DIAGNOSTICS, TAGS
 
 
 class RenderError(BaseException):
@@ -26,7 +25,7 @@ class Contributor:
         ORCID url
     """
 
-    def __init__(self, name: str, institute: str, orcid: str = None):
+    def __init__(self, name: str, institute: str, orcid: Optional[str] = None):
         self.name = name
         self.institute = institute
         self.orcid = orcid
@@ -58,7 +57,7 @@ class Contributor:
             The contributor tags are defined in the authors section in
             ``config-references.yml``.
         """
-        mapping = TAGS['authors'][tag]
+        mapping = TAGS.get_tag_value(section='authors', tag=tag)
 
         name = ' '.join(reversed(mapping['name'].split(', ')))
         institute = mapping.get('institute', 'No affiliation')
@@ -152,7 +151,7 @@ class Reference:
             The bibtex tags resolved as ``esmvaltool/references/{tag}.bibtex``
             or the corresponding directory as defined by the diagnostics path.
         """
-        filename = Path(REFERENCES_PATH, f'{tag}.bibtex')
+        filename = DIAGNOSTICS.references / f'{tag}.bibtex'
         return cls(filename)
 
     def __repr__(self) -> str:
@@ -163,11 +162,11 @@ class Reference:
         """Return string representation."""
         return self.render(renderer='plaintext')
 
-    def _repr_markdown_(self) -> str:
+    def _repr_html_(self) -> str:
         """Represent using markdown renderer in a notebook environment."""
-        return self.render(renderer='markdown')
+        return self.render(renderer='html')
 
-    def render(self, renderer: str = 'plaintext') -> str:
+    def render(self, renderer: str = 'html') -> str:
         """Render the reference.
 
         Parameters
@@ -183,10 +182,11 @@ class Reference:
         """
         style = 'plain'  # alpha, plain, unsrt, unsrtalpha
         backend = pybtex.plugin.find_plugin('pybtex.backends', renderer)()
-        style = pybtex.plugin.find_plugin('pybtex.style.formatting', style)()
+        formatter = pybtex.plugin.find_plugin('pybtex.style.formatting',
+                                              style)()
 
         try:
-            formatter = style.format_entry(self._key, self._entry)
+            formatter = formatter.format_entry(self._key, self._entry)
             rendered = formatter.text.render(backend)
         except Exception as err:
             raise RenderError(
