@@ -1,5 +1,4 @@
 """API for handing recipe output."""
-
 import base64
 import logging
 from collections.abc import Mapping
@@ -8,7 +7,7 @@ from typing import Optional, Tuple, Type
 
 import iris
 
-from .config import Session
+from ..config._config import TASKSEP
 from .recipe_info import RecipeInfo
 from .recipe_metadata import Contributor, Reference
 from .templates import get_template
@@ -26,8 +25,10 @@ class TaskOutput:
     files : dict
         Mapping of the filenames with the associated attributes.
     """
+
     def __init__(self, name: str, files: dict):
         self.name = name
+        self.title = name.replace('_', ' ').replace(TASKSEP, ': ').title()
         self.files = tuple(
             OutputFile.create(filename, attributes)
             for filename, attributes in files.items())
@@ -72,6 +73,36 @@ class TaskOutput:
         return cls(name=task.name, files=product_attributes)
 
 
+class DiagnosticOutput:
+    """Container for diagnostic output.
+
+    Parameters
+    ----------
+    name : str
+        Name of the diagnostic
+    title: str
+        Title of the diagnostic
+    description: str
+        Description of the diagnostic
+    task_output : :obj:`list` of :obj:`TaskOutput`
+        List of task output.
+    """
+
+    def __init__(self, name, task_output, title=None, description=None):
+        self.name = name
+        self.title = title if title else name.title()
+        self.description = description if description else ''
+        self.task_output = task_output
+
+    def __repr__(self):
+        """Return canonical string representation."""
+        indent = '  '
+        string = f'{self.name}:\n'
+        for task_output in self.task_output:
+            string += f'{indent}{task_output}\n'
+        return string
+
+
 class RecipeOutput(Mapping):
     """Container for recipe output.
 
@@ -80,15 +111,43 @@ class RecipeOutput(Mapping):
     task_output : dict
         Dictionary with recipe output grouped by task name. Each task value is
         a mapping of the filenames with the product attributes.
+
+    Attributes
+    ----------
+    diagnostics : dict
+        Dictionary with recipe output grouped by diagnostic.
+    info : RecipeInfo
+        The recipe used to create the output.
+    session : esmvalcore.config.Session
+        The session used to run the recipe.
     """
+
     def __init__(self, task_output: dict, session=None, info=None):
         self._raw_task_output = task_output
         self._task_output = {}
+        self.diagnostics = {}
         self.info = info
         self.session = session
+
+        # Group create task output and group by diagnostic
+        diagnostics: dict = {}
         for task_name, files in task_output.items():
-            self._task_output[task_name] = TaskOutput(name=task_name,
-                                                      files=files)
+            name = task_name.split(TASKSEP)[0]
+            if name not in diagnostics:
+                diagnostics[name] = []
+            task = TaskOutput(name=task_name, files=files)
+            self._task_output[task_name] = task
+            diagnostics[name].append(task)
+
+        # Create diagnostic output
+        for name, tasks in diagnostics.items():
+            diagnostic_info = info.data['diagnostics'][name]
+            self.diagnostics[name] = DiagnosticOutput(
+                name=name,
+                task_output=tasks,
+                title=diagnostic_info.get('title'),
+                description=diagnostic_info.get('description'),
+            )
 
     def __repr__(self):
         """Return canonical string representation."""
@@ -113,10 +172,7 @@ class RecipeOutput(Mapping):
         """Construct instance from `_recipe.Recipe` output.
 
         The core recipe format is not directly compatible with the API. This
-        constructor does the following:
-
-        1. Convert `config-user` dict to an instance of :obj:`Session`
-        2. Converts the raw recipe dict to :obj:`RecipeInfo`
+        constructor converts the raw recipe dict to :obj:`RecipeInfo`
 
         Parameters
         ----------
@@ -125,10 +181,9 @@ class RecipeOutput(Mapping):
         """
         task_output = recipe_output['task_output']
         recipe_data = recipe_output['recipe_data']
-        recipe_config = recipe_output['recipe_config']
+        session = recipe_output['session']
         recipe_filename = recipe_output['recipe_filename']
 
-        session = Session.from_config_user(recipe_config)
         info = RecipeInfo(recipe_data, filename=recipe_filename)
         info.resolve()
 
@@ -159,7 +214,7 @@ class RecipeOutput(Mapping):
         if not template:
             template = get_template(self.__class__.__name__ + '.j2')
         rendered = template.render(
-            tasks=self.values(),
+            diagnostics=self.diagnostics.values(),
             session=self.session,
             info=self.info,
         )
@@ -191,7 +246,7 @@ class OutputFile():
 
     kind: Optional[str] = None
 
-    def __init__(self, path: str, attributes: dict = None):
+    def __init__(self, path: str, attributes: Optional[dict] = None):
         if not attributes:
             attributes = {}
 
@@ -227,7 +282,7 @@ class OutputFile():
             self._references = tuple(Reference.from_tag(tag) for tag in tags)
         return self._references
 
-    def _get_derived_path(self, append: str, suffix: str = None):
+    def _get_derived_path(self, append: str, suffix: Optional[str] = None):
         """Return path of related files.
 
         Parameters
@@ -255,18 +310,18 @@ class OutputFile():
         """Return path of data citation info (txt format)."""
         return self._get_derived_path('_data_citation_info', '.txt')
 
-    @property
-    def provenance_svg_file(self):
-        """Return path of provenance file (svg format)."""
-        return self._get_derived_path('_provenance', '.svg')
-
     @property
     def provenance_xml_file(self):
         """Return path of provenance file (xml format)."""
         return self._get_derived_path('_provenance', '.xml')
 
     @classmethod
-    def create(cls, path: str, attributes: dict = None) -> 'OutputFile':
+    def create(
+        cls,
+        path: str,
+        attributes:
+        Optional[dict] = None,
+    ) -> 'OutputFile':
         """Construct new instances of OutputFile.
 
         Chooses a derived class if suitable.
