@@ -1,16 +1,18 @@
 """ESMValCore utilities."""
 
+import os
 import re
 from pathlib import Path
+from typing import Optional, Pattern, Tuple, Union
 
-from esmvalcore._config import DIAGNOSTICS_PATH
+from esmvalcore.config._diagnostics import DIAGNOSTICS
 
 from .recipe import Recipe
 
 
 class RecipeList(list):
     """Container for recipes."""
-    def find(self, query: str):
+    def find(self, query: Pattern[str]):
         """Search for recipes matching the search query or pattern.
 
         Searches in the description, authors and project information fields.
@@ -18,7 +20,7 @@ class RecipeList(list):
 
         Parameters
         ----------
-        query : str
+        query : str, Pattern
             String to search for, e.g. ``find_recipes('righi')`` will return
             all matching that author. Can be a `regex` pattern.
 
@@ -39,13 +41,13 @@ class RecipeList(list):
         return matches
 
 
-def get_all_recipes(subdir: str = None) -> list:
+def get_all_recipes(subdir: Optional[str] = None) -> list:
     """Return a list of all available recipes.
 
     Parameters
     ----------
-    subdir : str, optional
-        Sub-directory of the ``DIAGNOSTICS_PATH`` to look for
+    subdir : str
+        Sub-directory of the ``DIAGNOSTICS.path`` to look for
         recipes, e.g. ``get_all_recipes(subdir='examples')``.
 
     Returns
@@ -53,14 +55,14 @@ def get_all_recipes(subdir: str = None) -> list:
     RecipeList
         List of available recipes
     """
-    if not subdir:
+    if subdir is None:
         subdir = '**'
-    rootdir = Path(DIAGNOSTICS_PATH, 'recipes')
+    rootdir = DIAGNOSTICS.recipes
     files = rootdir.glob(f'{subdir}/*.yml')
     return RecipeList(Recipe(file) for file in files)
 
 
-def get_recipe(name: str) -> 'Recipe':
+def get_recipe(name: Union[os.PathLike, str]) -> Recipe:
     """Get a recipe by its name.
 
     The function looks first in the local directory, and second in the
@@ -83,12 +85,14 @@ def get_recipe(name: str) -> 'Recipe':
     FileNotFoundError
         If the name cannot be resolved to a recipe file.
     """
-    locations = Path(), Path(DIAGNOSTICS_PATH, 'recipes')
+    filenames: Tuple[Union[str, os.PathLike], ...]
 
-    if isinstance(name, Path):
-        filenames = (name, )
-    else:
+    locations = Path(), DIAGNOSTICS.recipes
+
+    if isinstance(name, str):
         filenames = (name, name + '.yml')
+    else:
+        filenames = (name, )
 
     for location in locations:
         for filename in filenames:
