@@ -1,8 +1,12 @@
 """Preprocessor module."""
+from __future__ import annotations
+
 import copy
 import inspect
 import logging
+from pathlib import Path
 from pprint import pformat
+from typing import Any, Iterable
 
 from iris.cube import Cube
 
@@ -10,7 +14,6 @@ from .._provenance import TrackedFile
 from .._task import BaseTask
 from ..cmor.check import cmor_check_data, cmor_check_metadata
 from ..cmor.fix import fix_data, fix_file, fix_metadata
-from ._ancillary_vars import add_fx_variables, remove_fx_variables
 from ._area import (
     area_statistics,
     extract_named_regions,
@@ -52,6 +55,12 @@ from ._regrid import (
     regrid,
 )
 from ._rolling_window import rolling_window_statistics
+from ._supplementary_vars import (
+    add_fx_variables,
+    add_supplementary_variables,
+    remove_fx_variables,
+    remove_supplementary_variables,
+)
 from ._time import (
     annual_statistics,
     anomalies,
@@ -89,8 +98,6 @@ __all__ = [
     'fix_file',
     # Load cubes from file
     'load',
-    # Derive variable
-    'derive',
     # Metadata reformatting/CMORization
     'fix_metadata',
     # Concatenate all cubes in one
@@ -101,8 +108,11 @@ __all__ = [
     # Data reformatting/CMORization
     'fix_data',
     'cmor_check_data',
-    # Load fx_variables in cube
+    # Attach ancillary variables and cell measures
     'add_fx_variables',
+    'add_supplementary_variables',
+    # Derive variable
+    'derive',
     # Time extraction (as defined in the preprocessor section)
     'extract_time',
     'extract_season',
@@ -176,7 +186,8 @@ __all__ = [
     'multi_model_statistics',
     # Bias calculation
     'bias',
-    # Remove fx_variables from cube
+    # Remove supplementary variables from cube
+    'remove_supplementary_variables',
     'remove_fx_variables',
     # Save to file
     'save',
@@ -204,8 +215,10 @@ By default, preprocessor functions are applied in this order.
 """
 
 # The order of initial and final steps cannot be configured
-INITIAL_STEPS = DEFAULT_ORDER[:DEFAULT_ORDER.index('add_fx_variables') + 1]
-FINAL_STEPS = DEFAULT_ORDER[DEFAULT_ORDER.index('remove_fx_variables'):]
+INITIAL_STEPS = DEFAULT_ORDER[:DEFAULT_ORDER.index(
+    'add_supplementary_variables') + 1]
+FINAL_STEPS = DEFAULT_ORDER[DEFAULT_ORDER.index(
+    'remove_supplementary_variables'):]
 
 MULTI_MODEL_FUNCTIONS = {
     'bias',
@@ -228,8 +241,9 @@ def check_preprocessor_settings(settings):
     for step in settings:
         if step not in DEFAULT_ORDER:
             raise ValueError(
-                "Unknown preprocessor function '{}', choose from: {}".format(
-                    step, ', '.join(DEFAULT_ORDER)))
+                f"Unknown preprocessor function '{step}', choose from: "
+                f"{', '.join(DEFAULT_ORDER)}"
+            )
 
         function = function = globals()[step]
         argspec = inspect.getfullargspec(function)
@@ -239,9 +253,10 @@ def check_preprocessor_settings(settings):
             invalid_args = set(settings[step]) - set(args)
             if invalid_args:
                 raise ValueError(
-                    "Invalid argument(s): {} encountered for preprocessor "
-                    "function {}. \nValid arguments are: [{}]".format(
-                        ', '.join(invalid_args), step, ', '.join(args)))
+                    f"Invalid argument(s): {', '.join(invalid_args)} "
+                    f"encountered for preprocessor function {step}. \n"
+                    f"Valid arguments are: [{', '.join(args)}]"
+                )
 
         # Check for missing arguments
         defaults = argspec.defaults
@@ -249,8 +264,9 @@ def check_preprocessor_settings(settings):
         missing_args = set(args[:end]) - set(settings[step])
         if missing_args:
             raise ValueError(
-                "Missing required argument(s) {} for preprocessor "
-                "function {}".format(missing_args, step))
+                f"Missing required argument(s) {missing_args} for "
+                f"preprocessor function {step}"
+            )
         # Final sanity check in case the above fails to catch a mistake
         try:
             signature = inspect.Signature.from_callable(function)
@@ -277,10 +293,9 @@ def _check_multi_model_settings(products):
             elif reference.settings[step] != settings:
                 raise ValueError(
                     "Unable to combine differing multi-dataset settings for "
-                    "{} and {}, {} and {}".format(reference.filename,
-                                                  product.filename,
-                                                  reference.settings[step],
-                                                  settings))
+                    f"{reference.filename} and {product.filename}, "
+                    f"{reference.settings[step]} and {settings}"
+                )
 
 
 def _get_multi_model_settings(products, step):
@@ -323,7 +338,7 @@ def _run_preproc_function(function, items, kwargs, input_files=None):
                         f"here; refer to the debug log for a full list)")
 
         # Make sure that the arguments are indexable
-        if isinstance(items, (PreprocessorFile, Cube, str)):
+        if isinstance(items, (PreprocessorFile, Cube, str, Path)):
             items = [items]
         if isinstance(items, set):
             items = list(items)
@@ -342,7 +357,14 @@ def _run_preproc_function(function, items, kwargs, input_files=None):
         raise
 
 
-def preprocess(items, step, input_files=None, **settings):
+def preprocess(
+    items,
+    step,
+    input_files=None,
+    output_file=None,
+    debug=False,
+    **settings
+):
     """Run preprocessor."""
     logger.debug("Running preprocessor step %s", step)
     function = globals()[step]
@@ -359,11 +381,17 @@ def preprocess(items, step, input_files=None, **settings):
 
     items = []
     for item in result:
-        if isinstance(item, (PreprocessorFile, Cube, str)):
+        if isinstance(item, (PreprocessorFile, Cube, str, Path)):
             items.append(item)
         else:
             items.extend(item)
 
+    if debug:
+        logger.debug("Result %s", items)
+        if all(isinstance(elem, Cube) for elem in items):
+            filename = _get_debug_filename(output_file, step)
+            save(items, filename)
+
     return items
 
 
@@ -371,7 +399,7 @@ def get_step_blocks(steps, order):
     """Group steps into execution blocks."""
     blocks = []
     prev_step_type = None
-    for step in order[order.index('load') + 1:order.index('save')]:
+    for step in order[len(INITIAL_STEPS):-len(FINAL_STEPS)]:
         if step in steps:
             step_type = step in MULTI_MODEL_FUNCTIONS
             if step_type is not prev_step_type:
@@ -385,66 +413,74 @@ def get_step_blocks(steps, order):
 class PreprocessorFile(TrackedFile):
     """Preprocessor output file."""
 
-    def __init__(self, attributes, settings, ancestors=None):
-        super().__init__(attributes['filename'], attributes, ancestors)
+    def __init__(
+        self,
+        filename: Path,
+        attributes: dict[str, Any] | None = None,
+        settings: dict[str, Any] | None = None,
+        datasets: list | None = None,
+    ):
+        if datasets is not None:
+            # Load data using a Dataset
+            input_files = []
+            for dataset in datasets:
+                input_files.extend(dataset.files)
+                for supplementary in dataset.supplementaries:
+                    input_files.extend(supplementary.files)
+            ancestors = [TrackedFile(f) for f in input_files]
+        else:
+            # Multimodel preprocessor functions set ancestors at runtime
+            # instead of here.
+            input_files = []
+            ancestors = []
 
+        self.datasets = datasets
+        self._cubes = None
+        self._input_files = input_files
+
+        # Set some preprocessor settings (move all defaults here?)
+        if settings is None:
+            settings = {}
         self.settings = copy.deepcopy(settings)
+        if attributes is None:
+            attributes = {}
+        attributes = copy.deepcopy(attributes)
         if 'save' not in self.settings:
             self.settings['save'] = {}
-        self.settings['save']['filename'] = self.filename
+        self.settings['save']['filename'] = filename
 
-        # self._input_files always contains the original input files;
-        # self.files may change in the preprocessing chain (e.g., by the step
-        # fix_file)
-        self._input_files = [a.filename for a in ancestors or ()]
-        self.files = copy.deepcopy(self._input_files)
+        attributes['filename'] = filename
 
-        self._cubes = None
-        self._prepared = False
-
-    def _input_files_for_log(self):
-        """Do not log input files twice in output log."""
-        if self.files == self._input_files:
-            return None
-        return self._input_files
+        super().__init__(
+            filename=filename,
+            attributes=attributes,
+            ancestors=ancestors,
+        )
 
     def check(self):
         """Check preprocessor settings."""
         check_preprocessor_settings(self.settings)
 
-    def apply(self, step, debug=False):
+    def apply(self, step: str, debug: bool = False):
         """Apply preprocessor step to product."""
         if step not in self.settings:
             raise ValueError(
-                "PreprocessorFile {} has no settings for step {}".format(
-                    self, step))
+                f"PreprocessorFile {self} has no settings for step {step}"
+            )
         self.cubes = preprocess(self.cubes, step,
                                 input_files=self._input_files,
+                                output_file=self.filename,
+                                debug=debug,
                                 **self.settings[step])
-        if debug:
-            logger.debug("Result %s", self.cubes)
-            filename = _get_debug_filename(self.filename, step)
-            save(self.cubes, filename)
-
-    def prepare(self):
-        """Apply preliminary file operations on product."""
-        if not self._prepared:
-            for step in DEFAULT_ORDER[:DEFAULT_ORDER.index('load')]:
-                if step in self.settings:
-                    self.files = preprocess(
-                        self.files, step,
-                        input_files=self._input_files_for_log(),
-                        **self.settings[step])
-            self._prepared = True
 
     @property
     def cubes(self):
         """Cubes."""
-        if self.is_closed:
-            self.prepare()
-            self._cubes = preprocess(self.files, 'load',
-                                     input_files=self._input_files_for_log(),
-                                     **self.settings.get('load', {}))
+        if self._cubes is None:
+            callback = self.settings.get('load', {}).get('callback')
+            self._cubes = [
+                ds._load_with_callback(callback) for ds in self.datasets
+            ]
         return self._cubes
 
     @cubes.setter
@@ -453,27 +489,54 @@ class PreprocessorFile(TrackedFile):
 
     def save(self):
         """Save cubes to disk."""
-        self.files = preprocess(self._cubes, 'save',
-                                input_files=self._input_files,
-                                **self.settings['save'])
-        self.files = preprocess(self.files, 'cleanup',
-                                input_files=self._input_files,
-                                **self.settings.get('cleanup', {}))
+        preprocess(self._cubes,
+                   'save',
+                   input_files=self._input_files,
+                   **self.settings['save'])
+        if 'cleanup' in self.settings:
+            preprocess([],
+                       'cleanup',
+                       input_files=self._input_files,
+                       **self.settings['cleanup'])
 
     def close(self):
         """Close the file."""
         if self._cubes is not None:
+            self._update_attributes()
             self.save()
             self._cubes = None
             self.save_provenance()
 
+    def _update_attributes(self):
+        """Update product attributes from cube metadata."""
+        if not self._cubes:
+            return
+        ref_cube = self._cubes[0]
+
+        # Names
+        names = {
+            'standard_name': 'standard_name',
+            'long_name': 'long_name',
+            'var_name': 'short_name',
+        }
+        for (name_in, name_out) in names.items():
+            cube_val = getattr(ref_cube, name_in)
+            self.attributes[name_out] = '' if cube_val is None else cube_val
+
+        # Units
+        self.attributes['units'] = str(ref_cube.units)
+
+        # Frequency
+        if 'frequency' in ref_cube.attributes:
+            self.attributes['frequency'] = ref_cube.attributes['frequency']
+
     @property
     def is_closed(self):
         """Check if the file is closed."""
         return self._cubes is None
 
     def _initialize_entity(self):
-        """Initialize the entity representing the file."""
+        """Initialize the provenance entity representing the file."""
         super()._initialize_entity()
         settings = {
             'preprocessor:' + k: str(v)
@@ -504,10 +567,6 @@ class PreprocessorFile(TrackedFile):
         return '_'.join(identifier)
 
 
-# TODO: use a custom ProductSet that raises an exception if you try to
-# add the same Product twice
-
-
 def _apply_multimodel(products, step, debug):
     """Apply multi model step to products."""
     settings, exclude = _get_multi_model_settings(products, step)
@@ -532,16 +591,15 @@ class PreprocessingTask(BaseTask):
 
     def __init__(
         self,
-        products,
-        ancestors=None,
-        name='',
-        order=DEFAULT_ORDER,
-        debug=None,
-        write_ncl_interface=False,
+        products: Iterable[PreprocessorFile],
+        name: str = '',
+        order: Iterable[str] = DEFAULT_ORDER,
+        debug: bool | None = None,
+        write_ncl_interface: bool = False,
     ):
         """Initialize."""
         _check_multi_model_settings(products)
-        super().__init__(ancestors=ancestors, name=name, products=products)
+        super().__init__(name=name, products=products)
         self.order = list(order)
         self.debug = debug
         self.write_ncl_interface = write_ncl_interface
@@ -596,6 +654,8 @@ class PreprocessingTask(BaseTask):
             for product in self.products for step in product.settings
         }
         blocks = get_step_blocks(steps, self.order)
+
+        saved = set()
         for block in blocks:
             logger.debug("Running block %s", block)
             if block[0] in MULTI_MODEL_FUNCTIONS:
@@ -609,10 +669,15 @@ class PreprocessingTask(BaseTask):
                         if step in product.settings:
                             product.apply(step, self.debug)
                     if block == blocks[-1]:
+                        product.cubes  # pylint: disable=pointless-statement
                         product.close()
+                        saved.add(product.filename)
 
         for product in self.products:
-            product.close()
+            if product.filename not in saved:
+                product.cubes  # pylint: disable=pointless-statement
+                product.close()
+
         metadata_files = write_metadata(self.products,
                                         self.write_ncl_interface)
         return metadata_files
@@ -623,13 +688,15 @@ class PreprocessingTask(BaseTask):
             step for step in self.order
             if any(step in product.settings for product in self.products)
         ]
-        products = '\n\n'.join('\n'.join([str(p), pformat(p.settings)])
-                               for p in self.products)
-        txt = "{}: {}\norder: {}\n{}\n{}".format(
-            self.__class__.__name__,
-            self.name,
-            order,
-            products,
+        products = '\n\n'.join('\n'.join([
+            str(p),
+            'input files: ' + pformat(p._input_files),
+            'settings: ' + pformat(p.settings),
+        ]) for p in self.products)
+        txt = "\n".join([
+            f"{self.__class__.__name__}: {self.name}",
+            f"order: {order}",
+            f"{products}",
             self.print_ancestors(),
-        )
+        ])
         return txt
