@@ -14,16 +14,17 @@ import shapely.ops
 from dask import array as da
 from iris.exceptions import CoordinateNotFoundError
 
-from ._ancillary_vars import (
-    add_ancillary_variable,
-    add_cell_measure,
-    remove_fx_variables,
-)
 from ._shared import (
     get_iris_analysis_operation,
     guess_bounds,
     operator_accept_weights,
 )
+from ._supplementary_vars import (
+    add_ancillary_variable,
+    add_cell_measure,
+    register_supplementaries,
+    remove_supplementary_variables,
+)
 
 logger = logging.getLogger(__name__)
 
@@ -202,6 +203,10 @@ def compute_area_weights(cube):
     return weights
 
 
+@register_supplementaries(
+    variables=['areacella', 'areacello'],
+    required='prefer_at_least_one',
+)
 def area_statistics(cube, operator):
     """Apply a statistical operator in the horizontal direction.
 
@@ -235,7 +240,11 @@ def area_statistics(cube, operator):
     Parameters
     ----------
         cube: iris.cube.Cube
-            Input cube.
+            Input cube. The input cube should have a
+            :class:`iris.coords.CellMeasure` named ``'cell_area'``, unless it
+            has regular 1D latitude and longitude coordinates so the cell areas
+            can be computed using
+            :func:`iris.analysis.cartography.area_weights`.
         operator: str
             The operation, options: mean, median, min, max, std_dev, sum,
             variance, rms.
@@ -656,7 +665,7 @@ def _mask_cube(cube, selections):
     cubelist = iris.cube.CubeList()
     for id_, select in selections.items():
         _cube = cube.copy()
-        remove_fx_variables(_cube)
+        remove_supplementary_variables(_cube)
         _cube.add_aux_coord(
             iris.coords.AuxCoord(id_, units='no_unit', long_name="shape_id"))
         select = da.broadcast_to(select, _cube.shape)
