@@ -3,16 +3,19 @@ import copy
 import logging
 import os
 import shutil
+import warnings
 from itertools import groupby
 from warnings import catch_warnings, filterwarnings
 
 import iris
 import iris.aux_factory
 import iris.exceptions
-import numpy as np
 import yaml
 from cf_units import suppress_errors
 
+from esmvalcore.exceptions import ESMValCoreDeprecationWarning
+from esmvalcore.iris_helpers import merge_cube_attributes
+
 from .._task import write_ncl_settings
 from ._time import extract_time
 
@@ -120,6 +123,9 @@ def load(file, callback=None, ignore_warnings=None):
         File to be loaded.
     callback: callable or None, optional (default: None)
         Callback function passed to :func:`iris.load_raw`.
+
+        .. deprecated:: 2.8.0
+            This argument will be removed in 2.10.0.
     ignore_warnings: list of dict or None, optional (default: None)
         Keyword arguments passed to :func:`warnings.filterwarnings` used to
         ignore warnings issued by :func:`iris.load_raw`. Each list element
@@ -135,6 +141,14 @@ def load(file, callback=None, ignore_warnings=None):
     ValueError
         Cubes are empty.
     """
+    if not (callback is None or callback == 'default'):
+        msg = ("The argument `callback` has been deprecated in "
+               "ESMValCore version 2.8.0 and is scheduled for removal in "
+               "version 2.10.0.")
+        warnings.warn(msg, ESMValCoreDeprecationWarning)
+    if callback == 'default':
+        callback = concatenate_callback
+    file = str(file)
     logger.debug("Loading:\n%s", file)
     if ignore_warnings is None:
         ignore_warnings = []
@@ -173,21 +187,6 @@ def load(file, callback=None, ignore_warnings=None):
     return raw_cubes
 
 
-def _fix_cube_attributes(cubes):
-    """Unify attributes of different cubes to allow concatenation."""
-    attributes = {}
-    for cube in cubes:
-        for (attr, val) in cube.attributes.items():
-            if attr not in attributes:
-                attributes[attr] = val
-            else:
-                if not np.array_equal(val, attributes[attr]):
-                    attributes[attr] = '{};{}'.format(str(attributes[attr]),
-                                                      str(val))
-    for cube in cubes:
-        cube.attributes = attributes
-
-
 def _by_two_concatenation(cubes):
     """Perform a by-2 concatenation to avoid gaps."""
     concatenated = iris.cube.CubeList(cubes).concatenate()
@@ -225,7 +224,7 @@ def concatenate(cubes):
     if len(cubes) == 1:
         return cubes[0]
 
-    _fix_cube_attributes(cubes)
+    merge_cube_attributes(cubes)
 
     if len(cubes) > 1:
         # order cubes by first time point
@@ -348,7 +347,35 @@ def _get_debug_filename(filename, step):
 
 
 def cleanup(files, remove=None):
-    """Clean up after running the preprocessor."""
+    """Clean up after running the preprocessor.
+
+    Warning
+    -------
+    .. deprecated:: 2.8.0
+        This function is no longer used and has been deprecated since
+        ESMValCore version 2.8.0. It is scheduled for removal in version
+        2.10.0.
+
+    Parameters
+    ----------
+    files: list of Path
+        Preprocessor output files (will not be removed if not in `removed`).
+    remove: list of Path or None, optional (default: None)
+        Files or directories to remove.
+
+    Returns
+    -------
+    list of Path
+        Preprocessor output files.
+
+    """
+    deprecation_msg = (
+        "The preprocessor function `cleanup` has been deprecated in "
+        "ESMValCore version 2.8.0 and is scheduled for removal in version "
+        "2.10.0."
+    )
+    warnings.warn(deprecation_msg, ESMValCoreDeprecationWarning)
+
     if remove is None:
         remove = []
 
