@@ -1,10 +1,8 @@
-"""
-Mask module.
+"""Mask module.
 
-Module that performs a number of masking
-operations that include: masking with ancillary variables, masking with
-Natural Earth shapefiles (land or ocean), masking on thresholds,
-missing values masking.
+Module that performs a number of masking operations that include:
+masking with ancillary variables, masking with Natural Earth shapefiles
+(land or ocean), masking on thresholds, missing values masking.
 """
 
 import logging
@@ -18,6 +16,8 @@ import shapely.vectorized as shp_vect
 from iris.analysis import Aggregator
 from iris.util import rolling_window
 
+from ._supplementary_vars import register_supplementaries
+
 logger = logging.getLogger(__name__)
 
 
@@ -61,29 +61,34 @@ def _apply_fx_mask(fx_mask, var_data):
     return var_data
 
 
-def mask_landsea(cube, mask_out, always_use_ne_mask=False):
-    """
-    Mask out either land mass or sea (oceans, seas and lakes).
+@register_supplementaries(
+    variables=['sftlf', 'sftof'],
+    required='prefer_at_least_one',
+)
+def mask_landsea(cube, mask_out):
+    """Mask out either land mass or sea (oceans, seas and lakes).
 
     It uses dedicated ancillary variables (sftlf or sftof) or,
     in their absence, it applies a
-    Natural Earth mask (land or ocean contours).
+    `Natural Earth <https://www.naturalearthdata.com>`_ mask (land or ocean
+    contours).
     Note that the Natural Earth masks have different resolutions:
     10m for land, and 50m for seas.
-    These are more than enough for ESMValTool purposes.
+    These are more than enough for masking climate model data.
 
     Parameters
     ----------
     cube: iris.cube.Cube
-        data cube to be masked.
+        data cube to be masked. If the cube has an
+        :class:`iris.coords.AncillaryVariable` with standard name
+        ``'land_area_fraction'`` or ``'sea_area_fraction'`` that will be used.
+        If both are present, only the 'land_area_fraction' will be used. If the
+        ancillary variable is not available, the mask will be calculated from
+        Natural Earth shapefiles.
 
     mask_out: str
         either "land" to mask out land mass or "sea" to mask out seas.
 
-    always_use_ne_mask: bool, optional (default: False)
-        always apply Natural Earths mask, regardless if fx files are available
-        or not.
-
     Returns
     -------
     iris.cube.Cube
@@ -92,10 +97,10 @@ def mask_landsea(cube, mask_out, always_use_ne_mask=False):
     Raises
     ------
     ValueError
-        Error raised if masking on irregular grids is attempted.
+        Error raised if masking on irregular grids is attempted without
+        an ancillary variable.
         Irregular grids are not currently supported for masking
         with Natural Earth shapefile masks.
-
     """
     # Dict to store the Natural Earth masks
     cwd = os.path.dirname(__file__)
@@ -105,65 +110,58 @@ def mask_landsea(cube, mask_out, always_use_ne_mask=False):
         'sea': os.path.join(cwd, 'ne_masks/ne_50m_ocean.shp')
     }
 
-    if not always_use_ne_mask:
-        # preserve importance order: try stflf first then sftof
-        fx_cube = None
+    # preserve importance order: try stflf first then sftof
+    fx_cube = None
+    try:
+        fx_cube = cube.ancillary_variable('land_area_fraction')
+    except iris.exceptions.AncillaryVariableNotFoundError:
         try:
-            fx_cube = cube.ancillary_variable('land_area_fraction')
+            fx_cube = cube.ancillary_variable('sea_area_fraction')
         except iris.exceptions.AncillaryVariableNotFoundError:
-            try:
-                fx_cube = cube.ancillary_variable('sea_area_fraction')
-            except iris.exceptions.AncillaryVariableNotFoundError:
-                logger.debug(
-                    'Ancillary variables land/sea area fraction '
-                    'not found in cube. Check fx_file availability.')
+            logger.debug('Ancillary variables land/sea area fraction not '
+                         'found in cube. Check fx_file availability.')
 
-        if fx_cube:
-            landsea_mask = _get_fx_mask(
-                fx_cube.data, mask_out, fx_cube.var_name)
-            cube.data = _apply_fx_mask(landsea_mask, cube.data)
-            logger.debug("Applying land-sea mask: %s", fx_cube.var_name)
-        else:
-            if cube.coord('longitude').points.ndim < 2:
-                cube = _mask_with_shp(cube, shapefiles[mask_out], [
-                    0,
-                ])
-                logger.debug(
-                    "Applying land-sea mask from Natural Earth"
-                    " shapefile: \n%s", shapefiles[mask_out])
-            else:
-                msg = ("Use of shapefiles with irregular grids not "
-                       "yet implemented, land-sea mask not applied.")
-                raise ValueError(msg)
+    if fx_cube:
+        fx_cube_data = da.broadcast_to(fx_cube.core_data(), cube.shape)
+        landsea_mask = _get_fx_mask(fx_cube_data, mask_out,
+                                    fx_cube.var_name)
+        cube.data = _apply_fx_mask(landsea_mask, cube.data)
+        logger.debug("Applying land-sea mask: %s", fx_cube.var_name)
     else:
         if cube.coord('longitude').points.ndim < 2:
             cube = _mask_with_shp(cube, shapefiles[mask_out], [
                 0,
             ])
             logger.debug(
-                "Applying land-sea mask from Natural Earth"
-                " shapefile: \n%s", shapefiles[mask_out])
+                "Applying land-sea mask from Natural Earth shapefile: \n%s",
+                shapefiles[mask_out],
+            )
         else:
-            msg = ("Use of shapefiles with irregular grids not "
-                   "yet implemented, land-sea mask not applied.")
+            msg = ("Use of shapefiles with irregular grids not yet "
+                   "implemented, land-sea mask not applied.")
             raise ValueError(msg)
 
     return cube
 
 
+@register_supplementaries(
+    variables=['sftgif'],
+    required='require_at_least_one',
+)
 def mask_landseaice(cube, mask_out):
-    """
-    Mask out either landsea (combined) or ice.
+    """Mask out either landsea (combined) or ice.
 
-    Function that masks out either landsea (land and seas) or ice (Antarctica
-    and Greenland and some wee glaciers).
+    Function that masks out either landsea (land and seas) or ice (Antarctica,
+    Greenland and some glaciers).
 
     It uses dedicated ancillary variables (sftgif).
 
     Parameters
     ----------
     cube: iris.cube.Cube
-        data cube to be masked.
+        data cube to be masked. It should have an
+        :class:`iris.coords.AncillaryVariable` with standard name
+        ``'land_ice_area_fraction'``.
 
     mask_out: str
         either "landsea" to mask out landsea or "ice" to mask out ice.
@@ -177,7 +175,6 @@ def mask_landseaice(cube, mask_out):
     ------
     ValueError
         Error raised if landsea-ice mask not found as an ancillary variable.
-
     """
     # sftgif is the only one so far but users can set others
     fx_cube = None
@@ -187,7 +184,8 @@ def mask_landseaice(cube, mask_out):
         logger.debug('Ancillary variable land ice area fraction '
                      'not found in cube. Check fx_file availability.')
     if fx_cube:
-        landice_mask = _get_fx_mask(fx_cube.data, mask_out, fx_cube.var_name)
+        fx_cube_data = da.broadcast_to(fx_cube.core_data(), cube.shape)
+        landice_mask = _get_fx_mask(fx_cube_data, mask_out, fx_cube.var_name)
         cube.data = _apply_fx_mask(landice_mask, cube.data)
         logger.debug("Applying landsea-ice mask: sftgif")
     else:
@@ -197,9 +195,8 @@ def mask_landseaice(cube, mask_out):
     return cube
 
 
-def mask_glaciated(cube, mask_out):
-    """
-    Mask out glaciated areas.
+def mask_glaciated(cube, mask_out: str = "glaciated"):
+    """Mask out glaciated areas.
 
     It applies a Natural Earth mask. Note that for computational reasons
     only the 10 largest polygons are used for masking.
@@ -256,7 +253,7 @@ def _get_geometries_from_shp(shapefilename):
     """Get the mask geometries out from a shapefile."""
     reader = shpreader.Reader(shapefilename)
     # Index 0 grabs the lowest resolution mask (no zoom)
-    geometries = [contour for contour in reader.geometries()]
+    geometries = list(reader.geometries())
     if not geometries:
         msg = "Could not find any geometry in {}".format(shapefilename)
         raise ValueError(msg)
@@ -268,17 +265,16 @@ def _get_geometries_from_shp(shapefilename):
 
 
 def _mask_with_shp(cube, shapefilename, region_indices=None):
-    """
-    Apply a Natural Earth land/sea mask.
+    """Apply a Natural Earth land/sea mask.
 
-    Apply a pre-made land or sea mask that is extracted form a
-    Natural Earth shapefile (proprietary file format). The masking
-    process is performed by checking if any given (x, y) point from
-    the data cube lies within the desired geometries (eg land, sea) stored
-    in the shapefile (this is done via shapefle vectorization and is fast).
+    Apply a pre-made land or sea mask that is extracted form a Natural
+    Earth shapefile (proprietary file format). The masking process is
+    performed by checking if any given (x, y) point from the data cube
+    lies within the desired geometries (eg land, sea) stored in the
+    shapefile (this is done via shapefle vectorization and is fast).
     region_indices is a list of indices that the user will want to index
-    the regions on (select a region by its index as it is listed in
-    the shapefile).
+    the regions on (select a region by its index as it is listed in the
+    shapefile).
     """
     # Create the region
     regions = _get_geometries_from_shp(shapefilename)
@@ -328,8 +324,7 @@ def _mask_with_shp(cube, shapefilename, region_indices=None):
 
 
 def count_spells(data, threshold, axis, spell_length):
-    """
-    Count data occurrences.
+    """Count data occurrences.
 
     Define a function to perform the custom statistical operation.
     Note: in order to meet the requirements of iris.analysis.Aggregator,
@@ -360,14 +355,13 @@ def count_spells(data, threshold, axis, spell_length):
     -------
     int
         Number of counts.
-
     """
     if axis < 0:
         # just cope with negative axis numbers
         axis += data.ndim
     # Threshold the data to find the 'significant' points.
     if not threshold:
-        data_hits = data
+        data_hits = np.ones_like(data, dtype=bool)
     else:
         data_hits = data > float(threshold)
     # Make an array with data values "windowed" along the time axis.
@@ -388,8 +382,7 @@ def count_spells(data, threshold, axis, spell_length):
 
 
 def mask_above_threshold(cube, threshold):
-    """
-    Mask above a specific threshold value.
+    """Mask above a specific threshold value.
 
     Takes a value 'threshold' and masks off anything that is above
     it in the cube data. Values equal to the threshold are not masked.
@@ -406,15 +399,13 @@ def mask_above_threshold(cube, threshold):
     -------
     iris.cube.Cube
         thresholded cube.
-
     """
     cube.data = np.ma.masked_where(cube.data > threshold, cube.data)
     return cube
 
 
 def mask_below_threshold(cube, threshold):
-    """
-    Mask below a specific threshold value.
+    """Mask below a specific threshold value.
 
     Takes a value 'threshold' and masks off anything that is below
     it in the cube data. Values equal to the threshold are not masked.
@@ -430,15 +421,13 @@ def mask_below_threshold(cube, threshold):
     -------
     iris.cube.Cube
         thresholded cube.
-
     """
     cube.data = np.ma.masked_where(cube.data < threshold, cube.data)
     return cube
 
 
 def mask_inside_range(cube, minimum, maximum):
-    """
-    Mask inside a specific threshold range.
+    """Mask inside a specific threshold range.
 
     Takes a MINIMUM and a MAXIMUM value for the range, and masks off anything
     that's between the two in the cube data.
@@ -456,15 +445,13 @@ def mask_inside_range(cube, minimum, maximum):
     -------
     iris.cube.Cube
         thresholded cube.
-
     """
     cube.data = np.ma.masked_inside(cube.data, minimum, maximum)
     return cube
 
 
 def mask_outside_range(cube, minimum, maximum):
-    """
-    Mask outside a specific threshold range.
+    """Mask outside a specific threshold range.
 
     Takes a MINIMUM and a MAXIMUM value for the range, and masks off anything
     that's outside the two in the cube data.
@@ -482,7 +469,6 @@ def mask_outside_range(cube, minimum, maximum):
     -------
     iris.cube.Cube
         thresholded cube.
-
     """
     cube.data = np.ma.masked_outside(cube.data, minimum, maximum)
     return cube
@@ -554,7 +540,6 @@ def mask_multimodel(products):
         Datasets have different shapes.
     TypeError
         Invalid input data.
-
     """
     if not products:
         return products
@@ -584,8 +569,7 @@ def mask_fillvalues(products,
                     threshold_fraction,
                     min_value=None,
                     time_window=1):
-    """
-    Compute and apply a multi-dataset fillvalues mask.
+    """Compute and apply a multi-dataset fillvalues mask.
 
     Construct the mask that fills a certain time window with missing values
     if the number of values in that specific window is less than a given
@@ -619,7 +603,6 @@ def mask_fillvalues(products,
     ------
     NotImplementedError
         Implementation missing for data with higher dimensionality than 4.
-
     """
     combined_mask = None
 
@@ -628,8 +611,8 @@ def mask_fillvalues(products,
     for product in products:
         for cube in product.cubes:
             cube.data = np.ma.fix_invalid(cube.data, copy=False)
-            mask = _get_fillvalues_mask(cube, threshold_fraction,
-                                        min_value, time_window)
+            mask = _get_fillvalues_mask(cube, threshold_fraction, min_value,
+                                        time_window)
             if combined_mask is None:
                 combined_mask = np.zeros_like(mask)
             # Select only valid (not all masked) pressure levels
@@ -646,7 +629,8 @@ def mask_fillvalues(products,
                     used.add(product)
             else:
                 raise NotImplementedError(
-                    "Unable to handle {} dimensional data".format(n_dims))
+                    f"Unable to handle {n_dims} dimensional data"
+                )
 
     if np.any(combined_mask):
         logger.debug("Applying fillvalues mask")
@@ -662,21 +646,21 @@ def mask_fillvalues(products,
 
 
 def _get_fillvalues_mask(cube, threshold_fraction, min_value, time_window):
-    """
-    Compute the per-model missing values mask.
+    """Compute the per-model missing values mask.
 
-    Construct the mask that fills a certain time window with missing values
-    if the number of values in that specific window is less than a given
-    fractional threshold; it uses a custom iris Aggregator function that
-    aggregates the cube data by a given time window and counts the number of
-    valid (unmasked) data points within that window;
-    a simple value thresholding is also applied if needed.
+    Construct the mask that fills a certain time window with missing
+    values if the number of values in that specific window is less than
+    a given fractional threshold; it uses a custom iris Aggregator
+    function that aggregates the cube data by a given time window and
+    counts the number of valid (unmasked) data points within that
+    window; a simple value thresholding is also applied if needed.
     """
     # basic checks
     if threshold_fraction < 0 or threshold_fraction > 1.0:
         raise ValueError(
-            "Fraction of missing values {} should be between 0 and 1.0".format(
-                threshold_fraction))
+            f"Fraction of missing values {threshold_fraction} should be "
+            f"between 0 and 1.0"
+        )
     nr_time_points = len(cube.coord('time').points)
     if time_window > nr_time_points:
         msg = "Time window (in time units) larger than total time span. Stop."
