@@ -6,6 +6,7 @@ constructing seasonal and area averages.
 import copy
 import datetime
 import logging
+from typing import Union
 from warnings import filterwarnings
 
 import dask.array as da
@@ -14,10 +15,12 @@ import iris.coord_categorisation
 import iris.cube
 import iris.exceptions
 import iris.util
+import isodate
 import numpy as np
 from iris.time import PartialDateTime
 
-from esmvalcore.cmor.check import _get_time_bounds
+from esmvalcore.cmor.check import _get_next_month, _get_time_bounds
+from esmvalcore.iris_helpers import date2num
 
 from ._shared import get_iris_analysis_operation, operator_accept_weights
 
@@ -35,7 +38,7 @@ for _coord in (
     filterwarnings(
         'ignore',
         "Collapsing a non-contiguous coordinate. "
-        "Metadata may not be fully descriptive for '{0}'.".format(_coord),
+        f"Metadata may not be fully descriptive for '{_coord}'.",
         category=UserWarning,
         module='iris',
     )
@@ -76,13 +79,6 @@ def extract_time(cube, start_year, start_month, start_day, end_year, end_month,
     ValueError
         if time ranges are outside the cube time limits
     """
-    time_coord = cube.coord('time')
-    time_units = time_coord.units
-    if time_units.calendar == '360_day':
-        if start_day > 30:
-            start_day = 30
-        if end_day > 30:
-            end_day = 30
     t_1 = PartialDateTime(year=int(start_year),
                           month=int(start_month),
                           day=int(start_day))
@@ -90,35 +86,150 @@ def extract_time(cube, start_year, start_month, start_day, end_year, end_month,
                           month=int(end_month),
                           day=int(end_day))
 
-    constraint = iris.Constraint(time=lambda t: t_1 <= t.point < t_2)
+    return _extract_datetime(cube, t_1, t_2)
+
+
+def _parse_start_date(date):
+    """Parse start of the input `timerange` tag given in ISO 8601 format.
+
+    Returns a datetime.datetime object.
+    """
+    if date.startswith('P'):
+        start_date = isodate.parse_duration(date)
+    else:
+        try:
+            start_date = isodate.parse_datetime(date)
+        except isodate.isoerror.ISO8601Error:
+            start_date = isodate.parse_date(date)
+            start_date = datetime.datetime.combine(
+                start_date, datetime.time.min)
+    return start_date
+
+
+def _parse_end_date(date):
+    """Parse end of the input `timerange` given in ISO 8601 format.
+
+    Returns a datetime.datetime object.
+    """
+    if date.startswith('P'):
+        end_date = isodate.parse_duration(date)
+    else:
+        if len(date) == 4:
+            end_date = datetime.datetime(int(date) + 1, 1, 1, 0, 0, 0)
+        elif len(date) == 6:
+            month, year = _get_next_month(int(date[4:]), int(date[0:4]))
+            end_date = datetime.datetime(year, month, 1, 0, 0, 0)
+        else:
+            try:
+                end_date = isodate.parse_datetime(date)
+            except isodate.ISO8601Error:
+                end_date = isodate.parse_date(date)
+                end_date = datetime.datetime.combine(end_date,
+                                                     datetime.time.min)
+            end_date += datetime.timedelta(seconds=1)
+    return end_date
+
+
+def _duration_to_date(duration, reference, sign):
+    """Add or subtract a duration period to a reference datetime."""
+    date = reference + sign * duration
+    return date
+
+
+def _select_timeslice(
+    cube: iris.cube.Cube,
+    select: np.ndarray,
+) -> Union[iris.cube.Cube, None]:
+    """Slice a cube along its time axis."""
+    if select.any():
+        coord = cube.coord('time')
+        time_dims = cube.coord_dims(coord)
+        if time_dims:
+            time_dim = time_dims[0]
+            slices = tuple(select if i == time_dim else slice(None)
+                           for i in range(cube.ndim))
+            cube_slice = cube[slices]
+        else:
+            cube_slice = cube
+    else:
+        cube_slice = None
+    return cube_slice
+
+
+def _extract_datetime(
+    cube: iris.cube.Cube,
+    start_datetime: PartialDateTime,
+    end_datetime: PartialDateTime,
+) -> iris.cube.Cube:
+    """Extract a time range from a cube.
+
+    Given a time range passed in as a datetime.datetime object, it
+    returns a time-extracted cube with data only within the specified
+    time range with a resolution up to seconds..
+
+    Parameters
+    ----------
+    cube: iris.cube.Cube
+        input cube.
+    start_datetime: PartialDateTime
+        start datetime
+    end_datetime: PartialDateTime
+        end datetime
+
+    Returns
+    -------
+    iris.cube.Cube
+        Sliced cube.
+
+    Raises
+    ------
+    ValueError
+        if time ranges are outside the cube time limits
+    """
+    time_coord = cube.coord('time')
+    time_units = time_coord.units
+    if time_units.calendar == '360_day':
+        if isinstance(start_datetime.day, int) and start_datetime.day > 30:
+            start_datetime.day = 30
+        if isinstance(end_datetime.day, int) and end_datetime.day > 30:
+            end_datetime.day = 30
+
+    if not cube.coord_dims(time_coord):
+        constraint = iris.Constraint(
+            time=lambda t: start_datetime <= t.point < end_datetime)
+        cube_slice = cube.extract(constraint)
+    else:
+        # Convert all time points to dates at once, this is much faster
+        # than using a constraint.
+        dates = time_coord.units.num2date(time_coord.points)
+        select = (dates >= start_datetime) & (dates < end_datetime)
+        cube_slice = _select_timeslice(cube, select)
 
-    cube_slice = cube.extract(constraint)
     if cube_slice is None:
-        raise ValueError(
-            f"Time slice {start_year:0>4d}-{start_month:0>2d}-{start_day:0>2d}"
-            f" to {end_year:0>4d}-{end_month:0>2d}-{end_day:0>2d} is outside "
-            f"cube time bounds {time_coord.cell(0)} to {time_coord.cell(-1)}.")
 
-    # Issue when time dimension was removed when only one point as selected.
-    if cube_slice.ndim != cube.ndim:
-        if cube_slice.coord('time') == time_coord:
-            logger.debug('No change needed to time.')
-            return cube
+        def dt2str(time: PartialDateTime) -> str:
+            txt = f"{time.year}-{time.month:02d}-{time.day:02d}"
+            if any([time.hour, time.minute, time.second]):
+                txt += f" {time.hour:02d}:{time.minute:02d}:{time.second:02d}"
+            return txt
+        raise ValueError(
+            f"Time slice {dt2str(start_datetime)} "
+            f"to {dt2str(end_datetime)} is outside "
+            f"cube time bounds {time_coord.cell(0).point} to "
+            f"{time_coord.cell(-1).point}.")
 
     return cube_slice
 
 
-def clip_start_end_year(cube, start_year, end_year):
-    """Extract time range given by the dataset keys.
+def clip_timerange(cube, timerange):
+    """Extract time range with a resolution up to seconds.
 
     Parameters
     ----------
     cube : iris.cube.Cube
         Input cube.
-    start_year : int
-        Start year.
-    end_year : int
-        End year.
+    timerange : str
+        Time range in ISO 8601 format.
 
     Returns
     -------
@@ -130,7 +241,43 @@ def clip_start_end_year(cube, start_year, end_year):
     ValueError
         Time ranges are outside the cube's time limits.
     """
-    return extract_time(cube, start_year, 1, 1, end_year + 1, 1, 1)
+    start_date = timerange.split('/')[0]
+    start_date = _parse_start_date(start_date)
+
+    end_date = timerange.split('/')[1]
+    end_date = _parse_end_date(end_date)
+
+    if isinstance(start_date, isodate.duration.Duration):
+        start_date = _duration_to_date(start_date, end_date, sign=-1)
+    elif isinstance(start_date, datetime.timedelta):
+        start_date = _duration_to_date(start_date, end_date, sign=-1)
+        start_date -= datetime.timedelta(seconds=1)
+
+    if isinstance(end_date, isodate.duration.Duration):
+        end_date = _duration_to_date(end_date, start_date, sign=1)
+    elif isinstance(end_date, datetime.timedelta):
+        end_date = _duration_to_date(end_date, start_date, sign=1)
+        end_date += datetime.timedelta(seconds=1)
+
+    t_1 = PartialDateTime(
+        year=start_date.year,
+        month=start_date.month,
+        day=start_date.day,
+        hour=start_date.hour,
+        minute=start_date.minute,
+        second=start_date.second,
+    )
+
+    t_2 = PartialDateTime(
+        year=end_date.year,
+        month=end_date.month,
+        day=end_date.day,
+        hour=end_date.hour,
+        minute=end_date.minute,
+        second=end_date.second,
+    )
+
+    return _extract_datetime(cube, t_1, t_2)
 
 
 def extract_season(cube, season):
@@ -233,16 +380,51 @@ def get_time_weights(cube):
         Array of time weights for averaging.
     """
     time = cube.coord('time')
-    time_weights = time.bounds[..., 1] - time.bounds[..., 0]
-    time_weights = time_weights.squeeze()
-    if time_weights.shape == ():
-        time_weights = da.broadcast_to(time_weights, cube.shape)
-    else:
-        time_weights = iris.util.broadcast_to_shape(time_weights, cube.shape,
-                                                    cube.coord_dims('time'))
+    coord_dims = cube.coord_dims('time')
+
+    # Multidimensional time coordinates are not supported: In this case,
+    # weights cannot be simply calculated as difference between the bounds
+    if len(coord_dims) > 1:
+        raise ValueError(
+            f"Weighted statistical operations are not supported for "
+            f"{len(coord_dims):d}D time coordinates, expected "
+            f"0D or 1D")
+
+    # Extract 1D time weights (= lengths of time intervals)
+    time_weights = time.core_bounds()[:, 1] - time.core_bounds()[:, 0]
     return time_weights
 
 
+def _aggregate_time_fx(result_cube, source_cube):
+    time_dim = set(source_cube.coord_dims(source_cube.coord('time')))
+    if source_cube.cell_measures():
+        for measure in source_cube.cell_measures():
+            measure_dims = set(source_cube.cell_measure_dims(measure))
+            if time_dim.intersection(measure_dims):
+                logger.debug('Averaging time dimension in measure %s.',
+                             measure.var_name)
+                result_measure = da.mean(measure.core_data(),
+                                         axis=tuple(time_dim))
+                measure = measure.copy(result_measure)
+                measure_dims = tuple(measure_dims - time_dim)
+                result_cube.add_cell_measure(measure, measure_dims)
+
+    if source_cube.ancillary_variables():
+        for ancillary_var in source_cube.ancillary_variables():
+            ancillary_dims = set(
+                source_cube.ancillary_variable_dims(ancillary_var))
+            if time_dim.intersection(ancillary_dims):
+                logger.debug(
+                    'Averaging time dimension in ancillary variable %s.',
+                    ancillary_var.var_name)
+                result_ancillary_var = da.mean(ancillary_var.core_data(),
+                                               axis=tuple(time_dim))
+                ancillary_var = ancillary_var.copy(result_ancillary_var)
+                ancillary_dims = tuple(ancillary_dims - time_dim)
+                result_cube.add_ancillary_variable(ancillary_var,
+                                                   ancillary_dims)
+
+
 def hourly_statistics(cube, hours, operator='mean'):
     """Compute hourly statistics.
 
@@ -279,12 +461,14 @@ def hourly_statistics(cube, hours, operator='mean'):
         iris.coord_categorisation.add_year(cube, 'time')
 
     operator = get_iris_analysis_operation(operator)
-    cube = cube.aggregated_by(['hour_group', 'day_of_year', 'year'], operator)
+    result = cube.aggregated_by(['hour_group', 'day_of_year', 'year'],
+                                operator)
 
-    cube.remove_coord('hour_group')
-    cube.remove_coord('day_of_year')
-    cube.remove_coord('year')
-    return cube
+    result.remove_coord('hour_group')
+    result.remove_coord('day_of_year')
+    result.remove_coord('year')
+
+    return result
 
 
 def daily_statistics(cube, operator='mean'):
@@ -313,11 +497,11 @@ def daily_statistics(cube, operator='mean'):
         iris.coord_categorisation.add_year(cube, 'time')
 
     operator = get_iris_analysis_operation(operator)
-    cube = cube.aggregated_by(['day_of_year', 'year'], operator)
+    result = cube.aggregated_by(['day_of_year', 'year'], operator)
 
-    cube.remove_coord('day_of_year')
-    cube.remove_coord('year')
-    return cube
+    result.remove_coord('day_of_year')
+    result.remove_coord('year')
+    return result
 
 
 def monthly_statistics(cube, operator='mean'):
@@ -346,8 +530,9 @@ def monthly_statistics(cube, operator='mean'):
         iris.coord_categorisation.add_year(cube, 'time')
 
     operator = get_iris_analysis_operation(operator)
-    cube = cube.aggregated_by(['month_number', 'year'], operator)
-    return cube
+    result = cube.aggregated_by(['month_number', 'year'], operator)
+    _aggregate_time_fx(result, cube)
+    return result
 
 
 def seasonal_statistics(cube,
@@ -378,9 +563,9 @@ def seasonal_statistics(cube,
     iris.cube.Cube
         Seasonal statistic cube
     """
-    seasons = tuple([sea.upper() for sea in seasons])
+    seasons = tuple(sea.upper() for sea in seasons)
 
-    if any([len(sea) < 2 for sea in seasons]):
+    if any(len(sea) < 2 for sea in seasons):
         raise ValueError(
             f"Minimum of 2 month is required per Seasons: {seasons}.")
 
@@ -390,8 +575,8 @@ def seasonal_statistics(cube,
                                              name='clim_season',
                                              seasons=seasons)
     else:
-        old_seasons = list(set(cube.coord('clim_season').points))
-        if not all([osea in seasons for osea in old_seasons]):
+        old_seasons = sorted(set(cube.coord('clim_season').points))
+        if not all(osea in seasons for osea in old_seasons):
             raise ValueError(
                 f"Seasons {seasons} do not match prior season extraction "
                 f"{old_seasons}.")
@@ -404,7 +589,7 @@ def seasonal_statistics(cube,
 
     operator = get_iris_analysis_operation(operator)
 
-    cube = cube.aggregated_by(['clim_season', 'season_year'], operator)
+    result = cube.aggregated_by(['clim_season', 'season_year'], operator)
 
     # CMOR Units are days so we are safe to operate on days
     # Ranging on [29, 31] days makes this calendar-independent
@@ -431,8 +616,10 @@ def seasonal_statistics(cube,
 
         return [dt[0] <= dn <= dt[1] for dn, dt in zip(num_days, tar_days)]
 
-    full_seasons = spans_full_season(cube)
-    return cube[full_seasons]
+    full_seasons = spans_full_season(result)
+    result = result[full_seasons]
+    _aggregate_time_fx(result, cube)
+    return result
 
 
 def annual_statistics(cube, operator='mean'):
@@ -464,7 +651,9 @@ def annual_statistics(cube, operator='mean'):
 
     if not cube.coords('year'):
         iris.coord_categorisation.add_year(cube, 'time')
-    return cube.aggregated_by('year', operator)
+    result = cube.aggregated_by('year', operator)
+    _aggregate_time_fx(result, cube)
+    return result
 
 
 def decadal_statistics(cube, operator='mean'):
@@ -503,8 +692,9 @@ def decadal_statistics(cube, operator='mean'):
 
         iris.coord_categorisation.add_categorised_coord(
             cube, 'decade', 'time', get_decade)
-
-    return cube.aggregated_by('decade', operator)
+    result = cube.aggregated_by('decade', operator)
+    _aggregate_time_fx(result, cube)
+    return result
 
 
 def climate_statistics(cube,
@@ -560,6 +750,7 @@ def climate_statistics(cube,
         operator = get_iris_analysis_operation(operator)
         clim_cube = cube.aggregated_by(clim_coord, operator)
         clim_cube.remove_coord('time')
+        _aggregate_time_fx(clim_cube, cube)
         if clim_cube.coord(clim_coord.name()).is_monotonic():
             iris.util.promote_aux_coord_to_dim_coord(clim_cube,
                                                      clim_coord.name())
@@ -570,7 +761,7 @@ def climate_statistics(cube,
 
     new_dtype = clim_cube.dtype
     if original_dtype != new_dtype:
-        logger.warning(
+        logger.debug(
             "climate_statistics changed dtype from "
             "%s to %s, changing back", original_dtype, new_dtype)
         clim_cube.data = clim_cube.core_data().astype(original_dtype)
@@ -717,7 +908,8 @@ def regrid_time(cube, frequency):
         cube with converted time axis and units.
     """
     # standardize time points
-    time_c = [cell.point for cell in cube.coord('time').cells()]
+    coord = cube.coord('time')
+    time_c = coord.units.num2date(coord.points)
     if frequency == 'yr':
         time_cells = [datetime.datetime(t.year, 7, 1, 0, 0, 0) for t in time_c]
     elif frequency == 'mon':
@@ -735,18 +927,19 @@ def regrid_time(cube, frequency):
         ]
     elif frequency == '3hr':
         time_cells = [
-            datetime.datetime(t.year, t.month, t.day, t.hour - t.hour % 3, 0,
-                              0) for t in time_c
+            datetime.datetime(
+                t.year, t.month, t.day, t.hour - t.hour % 3, 0, 0)
+            for t in time_c
         ]
     elif frequency == '6hr':
         time_cells = [
-            datetime.datetime(t.year, t.month, t.day, t.hour - t.hour % 6, 0,
-                              0) for t in time_c
+            datetime.datetime(
+                t.year, t.month, t.day, t.hour - t.hour % 6, 0, 0)
+            for t in time_c
         ]
 
-    cube.coord('time').points = [
-        cube.coord('time').units.date2num(cl) for cl in time_cells
-    ]
+    coord = cube.coord('time')
+    cube.coord('time').points = date2num(time_cells, coord.units, coord.dtype)
 
     # uniformize bounds
     cube.coord('time').bounds = None
@@ -862,9 +1055,9 @@ def timeseries_filter(cube,
         if filter_type == 'lowpass':
             wgts = low_pass_weights(window, 1. / span)
     else:
-        raise NotImplementedError("Filter type {} not implemented, \
-            please choose one of {}".format(filter_type,
-                                            ", ".join(supported_filters)))
+        raise NotImplementedError(
+            f"Filter type {filter_type} not implemented, "
+            f"please choose one of {', '.join(supported_filters)}")
 
     # Apply filter
     aggregation_operator = get_iris_analysis_operation(filter_stats)
@@ -924,9 +1117,17 @@ def resample_hours(cube, interval, offset=0):
     if cube_period.total_seconds() / 3600 >= interval:
         raise ValueError(f"Data period ({cube_period}) should be lower than "
                          f"the interval ({interval})")
-    hours = range(0 + offset, 24, interval)
-    select_hours = iris.Constraint(time=lambda cell: cell.point.hour in hours)
-    return cube.extract(select_hours)
+    hours = [PartialDateTime(hour=h) for h in range(0 + offset, 24, interval)]
+    dates = time.units.num2date(time.points)
+    select = np.zeros(len(dates), dtype=bool)
+    for hour in hours:
+        select |= dates == hour
+    cube = _select_timeslice(cube, select)
+    if cube is None:
+        raise ValueError(
+            f"Time coordinate {dates} does not contain {hours} for {cube}")
+
+    return cube
 
 
 def resample_time(cube, month=None, day=None, hour=None):
@@ -967,14 +1168,12 @@ def resample_time(cube, month=None, day=None, hour=None):
     iris.cube.Cube
         Cube with the new frequency.
     """
-    def compare(cell):
-        date = cell.point
-        if month is not None and month != date.month:
-            return False
-        if day is not None and day != date.day:
-            return False
-        if hour is not None and hour != date.hour:
-            return False
-        return True
-
-    return cube.extract(iris.Constraint(time=compare))
+    time = cube.coord('time')
+    dates = time.units.num2date(time.points)
+    requested = PartialDateTime(month=month, day=day, hour=hour)
+    select = dates == requested
+    cube = _select_timeslice(cube, select)
+    if cube is None:
+        raise ValueError(
+            f"Time coordinate {dates} does not contain {requested} for {cube}")
+    return cube
