@@ -8,7 +8,17 @@ import numpy as np
 import pytest
 from cf_units import Unit
 
-from esmvalcore.cmor._fixes.cmip6.cesm2 import Cl, Cli, Clw, Tas, Tos
+from esmvalcore.cmor._fixes.cmip6.cesm2 import (
+    Cl,
+    Cli,
+    Clw,
+    Fgco2,
+    Omon,
+    Siconc,
+    Tas,
+    Tos,
+)
+from esmvalcore.cmor._fixes.common import SiconcFixScalarCoord
 from esmvalcore.cmor.fix import Fix
 from esmvalcore.cmor.table import get_var_info
 
@@ -51,6 +61,7 @@ AIR_PRESSURE_BOUNDS = np.array([[[[[0.0, 1.5],
                                    [7.0, 25.0]]]]])
 
 
+@pytest.mark.sequential
 @pytest.mark.skipif(sys.version_info < (3, 7, 6),
                     reason="requires python3.7.6 or newer")
 @unittest.mock.patch(
@@ -79,7 +90,9 @@ def test_cl_fix_file(mock_get_filepath, tmp_path, test_data_path):
                                                   'fixed_cesm2_cl.nc')
     fix = Cl(None)
     fixed_file = fix.fix_file(nc_path, tmp_path)
-    mock_get_filepath.assert_called_once_with(tmp_path, nc_path)
+    mock_get_filepath.assert_called_once_with(
+        tmp_path, nc_path, add_unique_suffix=False
+    )
     fixed_cubes = iris.load(fixed_file)
     assert len(fixed_cubes) == 2
     var_names = [cube.var_name for cube in fixed_cubes]
@@ -97,15 +110,21 @@ def test_cl_fix_file(mock_get_filepath, tmp_path, test_data_path):
 
 
 @pytest.fixture
-def cl_cube():
+def cl_cubes():
     """``cl`` cube."""
     time_coord = iris.coords.DimCoord(
         [0.0, 1.0], var_name='time', standard_name='time',
         units='days since 1850-01-01 00:00:00')
+    a_coord = iris.coords.AuxCoord(
+        [0.1, 0.2, 0.1], bounds=[[0.0, 0.15], [0.15, 0.25], [0.25, 0.0]],
+        var_name='a', units='1')
+    b_coord = iris.coords.AuxCoord(
+        [0.9, 0.3, 0.1], bounds=[[1.0, 0.8], [0.8, 0.25], [0.25, 0.0]],
+        var_name='b', units='1')
     lev_coord = iris.coords.DimCoord(
-        [0.0, 1.0, 2.0], var_name='lev',
-        standard_name='atmosphere_hybrid_sigma_pressure_coordinate', units='1',
-        attributes={'positive': 'up'})
+        [999.0, 99.0, 9.0], var_name='lev',
+        standard_name='atmosphere_hybrid_sigma_pressure_coordinate',
+        units='hPa', attributes={'positive': 'up'})
     lat_coord = iris.coords.DimCoord(
         [0.0, 1.0], var_name='lat', standard_name='latitude', units='degrees')
     lon_coord = iris.coords.DimCoord(
@@ -122,30 +141,23 @@ def cl_cube():
         standard_name='cloud_area_fraction_in_atmosphere_layer',
         units='%',
         dim_coords_and_dims=coord_specs,
+        aux_coords_and_dims=[(a_coord, 1), (b_coord, 1)],
     )
-    return cube
+    return iris.cube.CubeList([cube])
 
 
-def test_cl_fix_data(cl_cube):
-    """Test ``fix_data`` for ``cl``."""
-    fix = Cl(None)
-    out_cube = fix.fix_data(cl_cube)
-    assert out_cube.shape == cl_cube.shape
-    np.testing.assert_allclose(out_cube.data,
-                               [[[[8, 9],
-                                  [10, 11]],
-                                 [[4, 5],
-                                  [6, 7]],
-                                 [[0, 1],
-                                  [2, 3]]],
-                                [[[20, 21],
-                                  [22, 23]],
-                                 [[16, 17],
-                                  [18, 19]],
-                                 [[12, 13],
-                                  [14, 15]]]])
-    np.testing.assert_allclose(out_cube.coord(var_name='lev').points,
-                               [2.0, 1.0, 0.0])
+def test_cl_fix_metadata(cl_cubes):
+    """Test ``fix_metadata`` for ``cl``."""
+    vardef = get_var_info('CMIP6', 'Amon', 'cl')
+    fix = Cl(vardef)
+    out_cubes = fix.fix_metadata(cl_cubes)
+    out_cube = out_cubes.extract_cube(
+        'cloud_area_fraction_in_atmosphere_layer')
+    lev_coord = out_cube.coord(var_name='lev')
+    assert lev_coord.units == '1'
+    np.testing.assert_allclose(lev_coord.points, [1.0, 0.5, 0.2])
+    np.testing.assert_allclose(lev_coord.bounds,
+                               [[1.0, 0.95], [0.95, 0.5], [0.5, 0.0]])
 
 
 def test_get_cli_fix():
@@ -225,6 +237,34 @@ def tos_cubes():
     return iris.cube.CubeList([tos_cube])
 
 
+@pytest.fixture
+def thetao_cubes():
+    """Cubes to test fixes for ``thetao``."""
+    time_coord = iris.coords.DimCoord(
+        [0.0004, 1.09776], var_name='time', standard_name='time',
+        units='days since 1850-01-01 00:00:00')
+    lat_coord = iris.coords.DimCoord(
+        [0.0, 1.0], var_name='lat', standard_name='latitude', units='degrees')
+    lon_coord = iris.coords.DimCoord(
+        [0.0, 1.0], var_name='lon', standard_name='longitude', units='degrees')
+    lev_coord = iris.coords.DimCoord(
+        [500.0, 1000.0], bounds=[[2.5, 7.5], [7.5, 12.5]],
+        var_name='lev', standard_name=None, units='cm',
+        attributes={'positive': 'up'})
+    coord_specs = [
+        (time_coord, 0),
+        (lev_coord, 1),
+        (lat_coord, 2),
+        (lon_coord, 3),
+    ]
+    thetao_cube = iris.cube.Cube(
+        np.ones((2, 2, 2, 2)),
+        var_name='thetao',
+        dim_coords_and_dims=coord_specs,
+    )
+    return iris.cube.CubeList([thetao_cube])
+
+
 def test_get_tas_fix():
     """Test getting of fix."""
     fix = Fix.get_fixes('CMIP6', 'CESM2', 'Amon', 'tas')
@@ -233,8 +273,26 @@ def test_get_tas_fix():
 
 def test_get_tos_fix():
     """Test getting of fix."""
-    fix = Fix.get_fixes('CMIP6', 'CESM2', 'Amon', 'tos')
-    assert fix == [Tos(None)]
+    fix = Fix.get_fixes('CMIP6', 'CESM2', 'Omon', 'tos')
+    assert fix == [Tos(None), Omon(None)]
+
+
+def test_get_thetao_fix():
+    """Test getting of fix."""
+    fix = Fix.get_fixes('CMIP6', 'CESM2', 'Omon', 'thetao')
+    assert fix == [Omon(None)]
+
+
+def test_get_fgco2_fix():
+    """Test getting of fix."""
+    fix = Fix.get_fixes('CMIP6', 'CESM2', 'Omon', 'fgco2')
+    assert fix == [Fgco2(None), Omon(None)]
+
+
+def test_get_siconc_fix():
+    """Test getting of fix."""
+    fix = Fix.get_fixes('CMIP6', 'CESM2', 'SImon', 'siconc')
+    assert fix == [Siconc(None)]
 
 
 def test_tas_fix_metadata(tas_cubes):
@@ -271,3 +329,55 @@ def test_tos_fix_metadata(tos_cubes):
     assert out_cubes is tos_cubes
     for cube in out_cubes:
         np.testing.assert_equal(cube.coord("time").points, [0., 1.1])
+
+
+def test_thetao_fix_metadata(thetao_cubes):
+    """Test ``fix_metadata`` for ``thetao``."""
+    vardef = get_var_info('CMIP6', 'Omon', 'thetao')
+    fix = Omon(vardef)
+    out_cubes = fix.fix_metadata(thetao_cubes)
+    assert out_cubes is thetao_cubes
+    assert len(out_cubes) == 1
+    out_cube = out_cubes[0]
+
+    # Check metadata of depth coordinate
+    depth_coord = out_cube.coord('depth')
+    assert depth_coord.standard_name == 'depth'
+    assert depth_coord.var_name == 'lev'
+    assert depth_coord.long_name == 'ocean depth coordinate'
+    assert depth_coord.units == 'm'
+    assert depth_coord.attributes == {'positive': 'down'}
+
+    # Check values of depth coordinate
+    np.testing.assert_allclose(depth_coord.points, [5.0, 10.0])
+    np.testing.assert_allclose(depth_coord.bounds, [[2.5, 7.5], [7.5, 12.5]])
+
+
+def test_fgco2_fix_metadata():
+    """Test ``fix_metadata`` for ``fgco2``."""
+    vardef = get_var_info('CMIP6', 'Omon', 'fgco2')
+    cubes = iris.cube.CubeList([
+        iris.cube.Cube(0.0, var_name='fgco2'),
+    ])
+    fix = Fgco2(vardef)
+    out_cubes = fix.fix_metadata(cubes)
+    assert out_cubes is cubes
+    assert len(out_cubes) == 1
+    out_cube = out_cubes[0]
+
+    # Check depth coordinate
+    depth_coord = out_cube.coord('depth')
+    assert depth_coord.standard_name == 'depth'
+    assert depth_coord.var_name == 'depth'
+    assert depth_coord.long_name == 'depth'
+    assert depth_coord.units == 'm'
+    assert depth_coord.attributes == {'positive': 'down'}
+
+    # Check values of depth coordinate
+    np.testing.assert_allclose(depth_coord.points, 0.0)
+    assert depth_coord.bounds is None
+
+
+def test_siconc_fix():
+    """Test fix for ``siconc``."""
+    assert Siconc is SiconcFixScalarCoord
