@@ -9,10 +9,26 @@ from iris import NameConstraint
 from iris.coords import AuxCoord, DimCoord
 from iris.cube import Cube, CubeList
 
-from esmvalcore._config import get_extra_facets
-from esmvalcore.cmor._fixes.icon.icon import AllVars, Siconc, Siconca
+import esmvalcore.cmor._fixes.icon.icon
+from esmvalcore.cmor._fixes.icon._base_fixes import IconFix
+from esmvalcore.cmor._fixes.icon.icon import AllVars, Clwvi, Siconc, Siconca
 from esmvalcore.cmor.fix import Fix
 from esmvalcore.cmor.table import get_var_info
+from esmvalcore.config._config import get_extra_facets
+from esmvalcore.dataset import Dataset
+
+TEST_GRID_FILE_URI = (
+    'https://github.com/ESMValGroup/ESMValCore/raw/main/tests/integration/'
+    'cmor/_fixes/test_data/icon_grid.nc'
+)
+TEST_GRID_FILE_NAME = 'icon_grid.nc'
+
+
+@pytest.fixture(autouse=True)
+def tmp_cache_dir(monkeypatch, tmp_path):
+    """Use temporary path as cache directory for all tests in this module."""
+    monkeypatch.setattr(IconFix, 'CACHE_DIR', tmp_path)
+
 
 # Note: test_data_path is defined in tests/integration/cmor/_fixes/conftest.py
 
@@ -73,14 +89,41 @@ def cubes_2d_lat_lon_grid():
     return CubeList([cube])
 
 
-def get_allvars_fix(mip, short_name):
-    """Get member of fix class."""
-    vardef = get_var_info('ICON', mip, short_name)
-    extra_facets = get_extra_facets('ICON', 'ICON', mip, short_name, ())
-    fix = AllVars(vardef, extra_facets=extra_facets)
+def _get_fix(mip, short_name, fix_name):
+    """Load a fix from esmvalcore.cmor._fixes.icon.icon."""
+    dataset = Dataset(
+        project='ICON',
+        dataset='ICON',
+        mip=mip,
+        short_name=short_name,
+    )
+    extra_facets = get_extra_facets(dataset, ())
+    vardef = get_var_info(project='ICON', mip=mip, short_name=short_name)
+    cls = getattr(esmvalcore.cmor._fixes.icon.icon, fix_name)
+    fix = cls(vardef, extra_facets=extra_facets)
     return fix
 
 
+def get_fix(mip, short_name):
+    """Load a variable fix from esmvalcore.cmor._fixes.icon.icon."""
+    fix_name = short_name[0].upper() + short_name[1:]
+    return _get_fix(mip, short_name, fix_name)
+
+
+def get_allvars_fix(mip, short_name):
+    """Load the AllVars fix from esmvalcore.cmor._fixes.icon.icon."""
+    return _get_fix(mip, short_name, 'AllVars')
+
+
+def fix_metadata(cubes, mip, short_name):
+    """Fix metadata of cubes."""
+    fix = get_fix(mip, short_name)
+    cubes = fix.fix_metadata(cubes)
+    fix = get_allvars_fix(mip, short_name)
+    cubes = fix.fix_metadata(cubes)
+    return cubes
+
+
 def check_ta_metadata(cubes):
     """Check ta metadata."""
     assert len(cubes) == 1
@@ -189,6 +232,7 @@ def check_lat(cube):
     assert lat.standard_name == 'latitude'
     assert lat.long_name == 'latitude'
     assert lat.units == 'degrees_north'
+    assert lat.attributes == {}
     np.testing.assert_allclose(
         lat.points,
         [-45.0, -45.0, -45.0, -45.0, 45.0, 45.0, 45.0, 45.0],
@@ -219,22 +263,23 @@ def check_lon(cube):
     assert lon.standard_name == 'longitude'
     assert lon.long_name == 'longitude'
     assert lon.units == 'degrees_east'
+    assert lon.attributes == {}
     np.testing.assert_allclose(
         lon.points,
-        [-135.0, -45.0, 45.0, 135.0, -135.0, -45.0, 45.0, 135.0],
+        [225.0, 315.0, 45.0, 135.0, 225.0, 315.0, 45.0, 135.0],
         rtol=1e-5
     )
     np.testing.assert_allclose(
         lon.bounds,
         [
-            [-135.0, -90.0, -180.0],
-            [-45.0, 0.0, -90.0],
-            [45.0, 90.0, 0.0],
-            [135.0, 180.0, 90.0],
-            [-180.0, -90.0, -135.0],
-            [-90.0, 0.0, -45.0],
-            [0.0, 90.0, 45.0],
-            [90.0, 180.0, 135.0],
+            [0.0, 270.0, 180.0],
+            [0.0, 0.0, 270.0],
+            [0.0, 90.0, 0.0],
+            [0.0, 180.0, 90.0],
+            [180.0, 270.0, 0.0],
+            [270.0, 0.0, 0.0],
+            [0.0, 90.0, 0.0],
+            [90.0, 180.0, 0.0],
         ],
         rtol=1e-5
     )
@@ -242,11 +287,15 @@ def check_lon(cube):
 
 
 def check_lat_lon(cube):
-    """Check latitude, longitude and spatial index coordinates of cube."""
+    """Check latitude, longitude and mesh of cube."""
     lat = check_lat(cube)
     lon = check_lon(cube)
 
-    # Check spatial index coordinate
+    # Check that latitude and longitude are mesh coordinates
+    assert cube.coords('latitude', mesh_coords=True)
+    assert cube.coords('longitude', mesh_coords=True)
+
+    # Check dimensional coordinate describing the mesh
     assert cube.coords('first spatial index for variables stored on an '
                        'unstructured grid', dim_coords=True)
     i_coord = cube.coord('first spatial index for variables stored on an '
@@ -263,6 +312,131 @@ def check_lat_lon(cube):
     assert cube.coord_dims(lat) == cube.coord_dims(lon)
     assert cube.coord_dims(lat) == cube.coord_dims(i_coord)
 
+    # Check the mesh itself
+    assert cube.location == 'face'
+    mesh = cube.mesh
+    check_mesh(mesh)
+
+
+def check_mesh(mesh):
+    """Check the mesh."""
+    assert mesh is not None
+    assert mesh.var_name is None
+    assert mesh.standard_name is None
+    assert mesh.long_name is None
+    assert mesh.units == 'unknown'
+    assert mesh.attributes == {}
+    assert mesh.cf_role == 'mesh_topology'
+    assert mesh.topology_dimension == 2
+
+    # Check face coordinates
+    assert len(mesh.coords(include_faces=True)) == 2
+
+    mesh_face_lat = mesh.coord(include_faces=True, axis='y')
+    assert mesh_face_lat.var_name == 'lat'
+    assert mesh_face_lat.standard_name == 'latitude'
+    assert mesh_face_lat.long_name == 'latitude'
+    assert mesh_face_lat.units == 'degrees_north'
+    assert mesh_face_lat.attributes == {}
+    np.testing.assert_allclose(
+        mesh_face_lat.points,
+        [-45.0, -45.0, -45.0, -45.0, 45.0, 45.0, 45.0, 45.0],
+        rtol=1e-5
+    )
+    np.testing.assert_allclose(
+        mesh_face_lat.bounds,
+        [
+            [-90.0, 0.0, 0.0],
+            [-90.0, 0.0, 0.0],
+            [-90.0, 0.0, 0.0],
+            [-90.0, 0.0, 0.0],
+            [0.0, 0.0, 90.0],
+            [0.0, 0.0, 90.0],
+            [0.0, 0.0, 90.0],
+            [0.0, 0.0, 90.0],
+        ],
+        rtol=1e-5
+    )
+
+    mesh_face_lon = mesh.coord(include_faces=True, axis='x')
+    assert mesh_face_lon.var_name == 'lon'
+    assert mesh_face_lon.standard_name == 'longitude'
+    assert mesh_face_lon.long_name == 'longitude'
+    assert mesh_face_lon.units == 'degrees_east'
+    assert mesh_face_lon.attributes == {}
+    np.testing.assert_allclose(
+        mesh_face_lon.points,
+        [225.0, 315.0, 45.0, 135.0, 225.0, 315.0, 45.0, 135.0],
+        rtol=1e-5
+    )
+    np.testing.assert_allclose(
+        mesh_face_lon.bounds,
+        [
+            [0.0, 270.0, 180.0],
+            [0.0, 0.0, 270.0],
+            [0.0, 90.0, 0.0],
+            [0.0, 180.0, 90.0],
+            [180.0, 270.0, 0.0],
+            [270.0, 0.0, 0.0],
+            [0.0, 90.0, 0.0],
+            [90.0, 180.0, 0.0],
+        ],
+        rtol=1e-5
+    )
+
+    # Check node coordinates
+    assert len(mesh.coords(include_nodes=True)) == 2
+
+    mesh_node_lat = mesh.coord(include_nodes=True, axis='y')
+    assert mesh_node_lat.var_name == 'nlat'
+    assert mesh_node_lat.standard_name == 'latitude'
+    assert mesh_node_lat.long_name == 'node latitude'
+    assert mesh_node_lat.units == 'degrees_north'
+    assert mesh_node_lat.attributes == {}
+    np.testing.assert_allclose(
+        mesh_node_lat.points,
+        [-90.0, 0.0, 0.0, 0.0, 0.0, 90.0],
+        rtol=1e-5
+    )
+    assert mesh_node_lat.bounds is None
+
+    mesh_node_lon = mesh.coord(include_nodes=True, axis='x')
+    assert mesh_node_lon.var_name == 'nlon'
+    assert mesh_node_lon.standard_name == 'longitude'
+    assert mesh_node_lon.long_name == 'node longitude'
+    assert mesh_node_lon.units == 'degrees_east'
+    assert mesh_node_lon.attributes == {}
+    np.testing.assert_allclose(
+        mesh_node_lon.points,
+        [0.0, 180.0, 270.0, 0.0, 90, 0.0],
+        rtol=1e-5
+    )
+    assert mesh_node_lon.bounds is None
+
+    # Check connectivity
+    assert len(mesh.connectivities()) == 1
+    conn = mesh.connectivity()
+    assert conn.var_name is None
+    assert conn.standard_name is None
+    assert conn.long_name is None
+    assert conn.units == 'unknown'
+    assert conn.attributes == {}
+    assert conn.cf_role == 'face_node_connectivity'
+    assert conn.start_index == 1
+    assert conn.location_axis == 0
+    assert conn.shape == (8, 3)
+    np.testing.assert_array_equal(
+        conn.indices,
+        [[1, 3, 2],
+         [1, 4, 3],
+         [1, 5, 4],
+         [1, 2, 5],
+         [2, 3, 6],
+         [3, 4, 6],
+         [4, 5, 6],
+         [5, 2, 6]],
+    )
+
 
 def check_typesi(cube):
     """Check scalar typesi coordinate of cube."""
@@ -324,19 +498,27 @@ def test_areacello_fix(cubes_grid):
     check_lat_lon(cube)
 
 
-# Test clwvi (for extra_facets)
+# Test clwvi (for extra fix)
 
 
 def test_get_clwvi_fix():
     """Test getting of fix."""
     fix = Fix.get_fixes('ICON', 'ICON', 'Amon', 'clwvi')
-    assert fix == [AllVars(None)]
+    assert fix == [Clwvi(None), AllVars(None)]
 
 
-def test_clwvi_fix(cubes_2d):
+def test_clwvi_fix(cubes_regular_grid):
     """Test fix."""
-    fix = get_allvars_fix('Amon', 'clwvi')
-    fixed_cubes = fix.fix_metadata(cubes_2d)
+    cubes = CubeList([
+        cubes_regular_grid[0].copy(),
+        cubes_regular_grid[0].copy()
+    ])
+    cubes[0].var_name = 'cllvi'
+    cubes[1].var_name = 'clivi'
+    cubes[0].units = '1e3 kg m-2'
+    cubes[1].units = '1e3 kg m-2'
+
+    fixed_cubes = fix_metadata(cubes, 'Amon', 'clwvi')
 
     assert len(fixed_cubes) == 1
     cube = fixed_cubes[0]
@@ -347,6 +529,32 @@ def test_clwvi_fix(cubes_2d):
     assert cube.units == 'kg m-2'
     assert 'positive' not in cube.attributes
 
+    np.testing.assert_allclose(cube.data, [[[0.0, 2000.0], [4000.0, 6000.0]]])
+
+
+# Test lwp (for extra_facets)
+
+
+def test_get_lwp_fix():
+    """Test getting of fix."""
+    fix = Fix.get_fixes('ICON', 'ICON', 'AERmon', 'lwp')
+    assert fix == [AllVars(None)]
+
+
+def test_lwp_fix(cubes_2d):
+    """Test fix."""
+    fix = get_allvars_fix('AERmon', 'lwp')
+    fixed_cubes = fix.fix_metadata(cubes_2d)
+
+    assert len(fixed_cubes) == 1
+    cube = fixed_cubes[0]
+    assert cube.var_name == 'lwp'
+    assert cube.standard_name == ('atmosphere_mass_content_of_cloud_liquid_'
+                                  'water')
+    assert cube.long_name == 'Liquid Water Path'
+    assert cube.units == 'kg m-2'
+    assert 'positive' not in cube.attributes
+
     check_time(cube)
     check_lat_lon(cube)
 
@@ -411,13 +619,7 @@ def test_get_siconc_fix():
 
 def test_siconc_fix(cubes_2d):
     """Test fix."""
-    vardef = get_var_info('ICON', 'SImon', 'siconc')
-    extra_facets = get_extra_facets('ICON', 'ICON', 'SImon', 'siconc', ())
-    siconc_fix = Siconc(vardef, extra_facets=extra_facets)
-    allvars_fix = get_allvars_fix('SImon', 'siconc')
-
-    fixed_cubes = siconc_fix.fix_metadata(cubes_2d)
-    fixed_cubes = allvars_fix.fix_metadata(fixed_cubes)
+    fixed_cubes = fix_metadata(cubes_2d, 'SImon', 'siconc')
 
     cube = check_siconc_metadata(fixed_cubes, 'siconc',
                                  'Sea-Ice Area Percentage (Ocean Grid)')
@@ -439,13 +641,7 @@ def test_get_siconca_fix():
 
 def test_siconca_fix(cubes_2d):
     """Test fix."""
-    vardef = get_var_info('ICON', 'SImon', 'siconca')
-    extra_facets = get_extra_facets('ICON', 'ICON', 'SImon', 'siconca', ())
-    siconca_fix = Siconca(vardef, extra_facets=extra_facets)
-    allvars_fix = get_allvars_fix('SImon', 'siconca')
-
-    fixed_cubes = siconca_fix.fix_metadata(cubes_2d)
-    fixed_cubes = allvars_fix.fix_metadata(fixed_cubes)
+    fixed_cubes = fix_metadata(cubes_2d, 'SImon', 'siconca')
 
     cube = check_siconc_metadata(fixed_cubes, 'siconca',
                                  'Sea-Ice Area Percentage (Atmospheric Grid)')
@@ -544,7 +740,38 @@ def test_tas_scalar_height2m_already_present(cubes_2d):
     check_heightxm(cube, 2.0)
 
 
-def test_tas_dim_height2m_already_present(cubes_2d):
+def test_tas_dim_height2m_already_present(cubes_2d, monkeypatch):
+    """Test fix."""
+    fix = get_allvars_fix('Amon', 'tas')
+    monkeypatch.setitem(fix.extra_facets, 'ugrid', False)
+    fixed_cubes = fix.fix_metadata(cubes_2d)
+
+    cube = check_tas_metadata(fixed_cubes)
+
+    assert cube.mesh is None
+
+    assert cube.coords('first spatial index for variables stored on an '
+                       'unstructured grid', dim_coords=True)
+    i_coord = cube.coord('first spatial index for variables stored on an '
+                         'unstructured grid', dim_coords=True)
+    assert i_coord.var_name == 'i'
+    assert i_coord.standard_name is None
+    assert i_coord.long_name == ('first spatial index for variables stored on '
+                                 'an unstructured grid')
+    assert i_coord.units == '1'
+    np.testing.assert_allclose(i_coord.points, [0, 1, 2, 3, 4, 5, 6, 7])
+    assert i_coord.bounds is None
+
+    assert cube.coords('latitude', dim_coords=False)
+    assert cube.coords('longitude', dim_coords=False)
+    lat = cube.coord('latitude', dim_coords=False)
+    lon = cube.coord('longitude', dim_coords=False)
+    assert len(cube.coord_dims(lat)) == 1
+    assert cube.coord_dims(lat) == cube.coord_dims(lon)
+    assert cube.coord_dims(lat) == cube.coord_dims(i_coord)
+
+
+def test_tas_no_mesh(cubes_2d):
     """Test fix."""
     fix = get_allvars_fix('Amon', 'tas')
 
@@ -663,17 +890,14 @@ def test_2d_lat_lon_grid_fix(cubes_2d_lat_lon_grid):
 # Test fix with empty standard_name
 
 
-def test_empty_standard_name_fix(cubes_2d):
+def test_empty_standard_name_fix(cubes_2d, monkeypatch):
     """Test fix."""
+    fix = get_allvars_fix('Amon', 'tas')
     # We know that tas has a standard name, but this being native model output
     # there may be variables with no standard name. The code is designed to
     # handle this gracefully and here we test it with an artificial, but
     # realistic case.
-    vardef = get_var_info('ICON', 'Amon', 'tas')
-    original_standard_name = vardef.standard_name
-    vardef.standard_name = ''
-    extra_facets = get_extra_facets('ICON', 'ICON', 'Amon', 'tas', ())
-    fix = AllVars(vardef, extra_facets=extra_facets)
+    monkeypatch.setattr(fix.vardef, 'standard_name', '')
     fixed_cubes = fix.fix_metadata(cubes_2d)
 
     assert len(fixed_cubes) == 1
@@ -684,9 +908,6 @@ def test_empty_standard_name_fix(cubes_2d):
     assert cube.units == 'K'
     assert 'positive' not in cube.attributes
 
-    # Restore original standard_name of tas
-    vardef.standard_name = original_standard_name
-
 
 # Test automatic addition of missing coordinates
 
@@ -721,22 +942,14 @@ def test_add_time_fail():
         fix._add_time(cube, cubes)
 
 
-def test_add_latitude(cubes_2d, tmp_path):
+def test_add_latitude(cubes_2d):
     """Test fix."""
     # Remove latitude from tas cube to test automatic addition
     tas_cube = cubes_2d.extract_cube(NameConstraint(var_name='tas'))
     tas_cube.remove_coord('latitude')
-    tas_cube.attributes['grid_file_uri'] = (
-        'https://github.com/ESMValGroup/ESMValCore/raw/main/tests/'
-        'integration/cmor/_fixes/test_data/icon_grid.nc'
-    )
     cubes = CubeList([tas_cube])
     fix = get_allvars_fix('Amon', 'tas')
 
-    # Temporary overwrite default cache location for downloads
-    original_cache_dir = fix.CACHE_DIR
-    fix.CACHE_DIR = tmp_path
-
     assert len(fix._horizontal_grids) == 0
     fixed_cubes = fix.fix_metadata(cubes)
 
@@ -744,28 +957,17 @@ def test_add_latitude(cubes_2d, tmp_path):
     assert cube.shape == (1, 8)
     check_lat_lon(cube)
     assert len(fix._horizontal_grids) == 1
-    assert 'icon_grid.nc' in fix._horizontal_grids
-
-    # Restore cache location
-    fix.CACHE_DIR = original_cache_dir
+    assert TEST_GRID_FILE_NAME in fix._horizontal_grids
 
 
-def test_add_longitude(cubes_2d, tmp_path):
+def test_add_longitude(cubes_2d):
     """Test fix."""
     # Remove longitude from tas cube to test automatic addition
     tas_cube = cubes_2d.extract_cube(NameConstraint(var_name='tas'))
     tas_cube.remove_coord('longitude')
-    tas_cube.attributes['grid_file_uri'] = (
-        'https://github.com/ESMValGroup/ESMValCore/raw/main/tests/'
-        'integration/cmor/_fixes/test_data/icon_grid.nc'
-    )
     cubes = CubeList([tas_cube])
     fix = get_allvars_fix('Amon', 'tas')
 
-    # Temporary overwrite default cache location for downloads
-    original_cache_dir = fix.CACHE_DIR
-    fix.CACHE_DIR = tmp_path
-
     assert len(fix._horizontal_grids) == 0
     fixed_cubes = fix.fix_metadata(cubes)
 
@@ -773,29 +975,18 @@ def test_add_longitude(cubes_2d, tmp_path):
     assert cube.shape == (1, 8)
     check_lat_lon(cube)
     assert len(fix._horizontal_grids) == 1
-    assert 'icon_grid.nc' in fix._horizontal_grids
-
-    # Restore cache location
-    fix.CACHE_DIR = original_cache_dir
+    assert TEST_GRID_FILE_NAME in fix._horizontal_grids
 
 
-def test_add_latitude_longitude(cubes_2d, tmp_path):
+def test_add_latitude_longitude(cubes_2d):
     """Test fix."""
     # Remove latitude and longitude from tas cube to test automatic addition
     tas_cube = cubes_2d.extract_cube(NameConstraint(var_name='tas'))
     tas_cube.remove_coord('latitude')
     tas_cube.remove_coord('longitude')
-    tas_cube.attributes['grid_file_uri'] = (
-        'https://github.com/ESMValGroup/ESMValCore/raw/main/tests/'
-        'integration/cmor/_fixes/test_data/icon_grid.nc'
-    )
     cubes = CubeList([tas_cube])
     fix = get_allvars_fix('Amon', 'tas')
 
-    # Temporary overwrite default cache location for downloads
-    original_cache_dir = fix.CACHE_DIR
-    fix.CACHE_DIR = tmp_path
-
     assert len(fix._horizontal_grids) == 0
     fixed_cubes = fix.fix_metadata(cubes)
 
@@ -803,17 +994,16 @@ def test_add_latitude_longitude(cubes_2d, tmp_path):
     assert cube.shape == (1, 8)
     check_lat_lon(cube)
     assert len(fix._horizontal_grids) == 1
-    assert 'icon_grid.nc' in fix._horizontal_grids
-
-    # Restore cache location
-    fix.CACHE_DIR = original_cache_dir
+    assert TEST_GRID_FILE_NAME in fix._horizontal_grids
 
 
 def test_add_latitude_fail(cubes_2d):
     """Test fix."""
-    # Remove latitude from tas cube to test automatic addition
+    # Remove latitude and grid file attribute from tas cube to test automatic
+    # addition
     tas_cube = cubes_2d.extract_cube(NameConstraint(var_name='tas'))
     tas_cube.remove_coord('latitude')
+    tas_cube.attributes = {}
     cubes = CubeList([tas_cube])
     fix = get_allvars_fix('Amon', 'tas')
 
@@ -824,9 +1014,11 @@ def test_add_latitude_fail(cubes_2d):
 
 def test_add_longitude_fail(cubes_2d):
     """Test fix."""
-    # Remove longitude from tas cube to test automatic addition
+    # Remove longitude and grid file attribute from tas cube to test automatic
+    # addition
     tas_cube = cubes_2d.extract_cube(NameConstraint(var_name='tas'))
     tas_cube.remove_coord('longitude')
+    tas_cube.attributes = {}
     cubes = CubeList([tas_cube])
     fix = get_allvars_fix('Amon', 'tas')
 
@@ -841,8 +1033,7 @@ def test_add_coord_from_grid_file_fail_invalid_coord():
 
     msg = r"coord_name must be one of .* got 'invalid_coord_name'"
     with pytest.raises(ValueError, match=msg):
-        fix._add_coord_from_grid_file(mock.sentinel.cube, 'invalid_coord_name',
-                                      'invalid_target_name')
+        fix._add_coord_from_grid_file(mock.sentinel.cube, 'invalid_coord_name')
 
 
 def test_add_coord_from_grid_file_fail_no_url():
@@ -852,69 +1043,49 @@ def test_add_coord_from_grid_file_fail_no_url():
     msg = ("Cube does not contain the attribute 'grid_file_uri' necessary to "
            "download the ICON horizontal grid file")
     with pytest.raises(ValueError, match=msg):
-        fix._add_coord_from_grid_file(Cube(0), 'grid_latitude', 'latitude')
+        fix._add_coord_from_grid_file(Cube(0), 'latitude')
 
 
-def test_add_coord_from_grid_fail_no_unnamed_dim(cubes_2d, tmp_path):
+def test_add_coord_from_grid_fail_no_unnamed_dim(cubes_2d):
     """Test fix."""
     # Remove latitude from tas cube to test automatic addition
     tas_cube = cubes_2d.extract_cube(NameConstraint(var_name='tas'))
     tas_cube.remove_coord('latitude')
-    tas_cube.attributes['grid_file_uri'] = (
-        'https://github.com/ESMValGroup/ESMValCore/raw/main/tests/'
-        'integration/cmor/_fixes/test_data/icon_grid.nc'
-    )
     index_coord = DimCoord(np.arange(8), var_name='ncells')
     tas_cube.add_dim_coord(index_coord, 1)
     fix = get_allvars_fix('Amon', 'tas')
 
-    # Temporary overwrite default cache location for downloads
-    original_cache_dir = fix.CACHE_DIR
-    fix.CACHE_DIR = tmp_path
-
     msg = ("Cannot determine coordinate dimension for coordinate 'latitude', "
            "cube does not contain a single unnamed dimension")
     with pytest.raises(ValueError, match=msg):
-        fix._add_coord_from_grid_file(tas_cube, 'grid_latitude', 'latitude')
-
-    # Restore cache location
-    fix.CACHE_DIR = original_cache_dir
+        fix._add_coord_from_grid_file(tas_cube, 'latitude')
 
 
-def test_add_coord_from_grid_fail_two_unnamed_dims(cubes_2d, tmp_path):
+def test_add_coord_from_grid_fail_two_unnamed_dims(cubes_2d):
     """Test fix."""
     # Remove latitude from tas cube to test automatic addition
     tas_cube = cubes_2d.extract_cube(NameConstraint(var_name='tas'))
     tas_cube.remove_coord('latitude')
-    tas_cube.attributes['grid_file_uri'] = (
-        'https://github.com/ESMValGroup/ESMValCore/raw/main/tests/'
-        'integration/cmor/_fixes/test_data/icon_grid.nc'
-    )
     tas_cube = iris.util.new_axis(tas_cube)
     fix = get_allvars_fix('Amon', 'tas')
 
-    # Temporary overwrite default cache location for downloads
-    original_cache_dir = fix.CACHE_DIR
-    fix.CACHE_DIR = tmp_path
-
     msg = ("Cannot determine coordinate dimension for coordinate 'latitude', "
            "cube does not contain a single unnamed dimension")
     with pytest.raises(ValueError, match=msg):
-        fix._add_coord_from_grid_file(tas_cube, 'grid_latitude', 'latitude')
-
-    # Restore cache location
-    fix.CACHE_DIR = original_cache_dir
+        fix._add_coord_from_grid_file(tas_cube, 'latitude')
 
 
 @mock.patch('esmvalcore.cmor._fixes.icon._base_fixes.requests', autospec=True)
 def test_get_horizontal_grid_cached_in_dict(mock_requests):
     """Test fix."""
     cube = Cube(0, attributes={'grid_file_uri': 'cached_grid_url.nc'})
+    grid_cube = Cube(0)
     fix = get_allvars_fix('Amon', 'tas')
-    fix._horizontal_grids['cached_grid_url.nc'] = mock.sentinel.grid
+    fix._horizontal_grids['cached_grid_url.nc'] = grid_cube
 
     grid = fix.get_horizontal_grid(cube)
-    assert grid == mock.sentinel.grid
+    assert grid == grid_cube
+    assert grid is not grid_cube
     assert mock_requests.mock_calls == []
 
 
@@ -930,10 +1101,6 @@ def test_get_horizontal_grid_cached_in_file(mock_requests, tmp_path):
     grid_cube = Cube(0, var_name='grid')
     iris.save(grid_cube, str(tmp_path / 'grid_file.nc'))
 
-    # Temporary overwrite default cache location for downloads
-    original_cache_dir = fix.CACHE_DIR
-    fix.CACHE_DIR = tmp_path
-
     grid = fix.get_horizontal_grid(cube)
     assert isinstance(grid, CubeList)
     assert len(grid) == 1
@@ -942,16 +1109,10 @@ def test_get_horizontal_grid_cached_in_file(mock_requests, tmp_path):
     assert 'grid_file.nc' in fix._horizontal_grids
     assert mock_requests.mock_calls == []
 
-    # Restore cache location
-    fix.CACHE_DIR = original_cache_dir
 
-
-def test_get_horizontal_grid_cache_file_too_old(tmp_path):
+def test_get_horizontal_grid_cache_file_too_old(tmp_path, monkeypatch):
     """Test fix."""
-    cube = Cube(0, attributes={
-        'grid_file_uri': 'https://github.com/ESMValGroup/ESMValCore/raw/main/'
-                         'tests/integration/cmor/_fixes/test_data/'
-                         'icon_grid.nc'})
+    cube = Cube(0, attributes={'grid_file_uri': TEST_GRID_FILE_URI})
     fix = get_allvars_fix('Amon', 'tas')
     assert len(fix._horizontal_grids) == 0
 
@@ -961,41 +1122,38 @@ def test_get_horizontal_grid_cache_file_too_old(tmp_path):
 
     # Temporary overwrite default cache location for downloads and cache
     # validity duration
-    original_cache_dir = fix.CACHE_DIR
-    original_cache_validity = fix.CACHE_VALIDITY
-    fix.CACHE_DIR = tmp_path
-    fix.CACHE_VALIDITY = -1
+    monkeypatch.setattr(fix, 'CACHE_VALIDITY', -1)
 
     grid = fix.get_horizontal_grid(cube)
     assert isinstance(grid, CubeList)
-    assert len(grid) == 1
-    assert grid[0].var_name == 'cell_area'
+    assert len(grid) == 4
+    var_names = [cube.var_name for cube in grid]
+    assert 'cell_area' in var_names
+    assert 'dual_area' in var_names
+    assert 'vertex_index' in var_names
+    assert 'vertex_of_cell' in var_names
     assert len(fix._horizontal_grids) == 1
-    assert 'icon_grid.nc' in fix._horizontal_grids
-
-    # Restore cache location
-    fix.CACHE_DIR = original_cache_dir
-    fix.CACHE_VALIDITY = original_cache_validity
+    assert TEST_GRID_FILE_NAME in fix._horizontal_grids
 
 
 # Test with single-dimension cubes
 
 
-def test_only_time():
+def test_only_time(monkeypatch):
     """Test fix."""
+    fix = get_allvars_fix('Amon', 'ta')
     # We know that ta has dimensions time, plev19, latitude, longitude, but the
     # ICON CMORizer is designed to check for the presence of each dimension
     # individually. To test this, remove all but one dimension of ta to create
     # an artificial, but realistic test case.
-    vardef = get_var_info('ICON', 'Amon', 'ta')
-    original_dimensions = vardef.dimensions
-    vardef.dimensions = ['time']
-    extra_facets = get_extra_facets('ICON', 'ICON', 'Amon', 'ta', ())
-    fix = AllVars(vardef, extra_facets=extra_facets)
+    monkeypatch.setattr(fix.vardef, 'dimensions', ['time'])
 
     # Create cube with only a single dimension
-    time_coord = DimCoord([0.0, 1.0], var_name='time', standard_name='time',
-                          long_name='time', units='days since 1850-01-01')
+    time_coord = DimCoord([0.0, 1.0],
+                          var_name='time',
+                          standard_name='time',
+                          long_name='time',
+                          units='days since 1850-01-01')
     cubes = CubeList([
         Cube([1, 1], var_name='ta', units='K',
              dim_coords_and_dims=[(time_coord, 0)]),
@@ -1022,25 +1180,24 @@ def test_only_time():
     np.testing.assert_allclose(new_time_coord.bounds,
                                [[-0.5, 0.5], [0.5, 1.5]])
 
-    # Restore original dimensions of ta
-    vardef.dimensions = original_dimensions
+    # Check that no mesh has been created
+    assert cube.mesh is None
 
 
-def test_only_height():
+def test_only_height(monkeypatch):
     """Test fix."""
+    fix = get_allvars_fix('Amon', 'ta')
     # We know that ta has dimensions time, plev19, latitude, longitude, but the
     # ICON CMORizer is designed to check for the presence of each dimension
     # individually. To test this, remove all but one dimension of ta to create
     # an artificial, but realistic test case.
-    vardef = get_var_info('ICON', 'Amon', 'ta')
-    original_dimensions = vardef.dimensions
-    vardef.dimensions = ['plev19']
-    extra_facets = get_extra_facets('ICON', 'ICON', 'Amon', 'ta', ())
-    fix = AllVars(vardef, extra_facets=extra_facets)
+    monkeypatch.setattr(fix.vardef, 'dimensions', ['plev19'])
 
     # Create cube with only a single dimension
-    height_coord = DimCoord([1000.0, 100.0], var_name='height',
-                            standard_name='height', units='cm')
+    height_coord = DimCoord([1000.0, 100.0],
+                            var_name='height',
+                            standard_name='height',
+                            units='cm')
     cubes = CubeList([
         Cube([1, 1], var_name='ta', units='K',
              dim_coords_and_dims=[(height_coord, 0)]),
@@ -1067,24 +1224,23 @@ def test_only_height():
     np.testing.assert_allclose(new_height_coord.points, [1.0, 10.0])
     assert new_height_coord.bounds is None
 
-    # Restore original dimensions of ta
-    vardef.dimensions = original_dimensions
+    # Check that no mesh has been created
+    assert cube.mesh is None
 
 
-def test_only_latitude():
+def test_only_latitude(monkeypatch):
     """Test fix."""
+    fix = get_allvars_fix('Amon', 'ta')
     # We know that ta has dimensions time, plev19, latitude, longitude, but the
     # ICON CMORizer is designed to check for the presence of each dimension
     # individually. To test this, remove all but one dimension of ta to create
     # an artificial, but realistic test case.
-    vardef = get_var_info('ICON', 'Amon', 'ta')
-    original_dimensions = vardef.dimensions
-    vardef.dimensions = ['latitude']
-    extra_facets = get_extra_facets('ICON', 'ICON', 'Amon', 'ta', ())
-    fix = AllVars(vardef, extra_facets=extra_facets)
+    monkeypatch.setattr(fix.vardef, 'dimensions', ['latitude'])
 
     # Create cube with only a single dimension
-    lat_coord = DimCoord([0.0, 10.0], var_name='lat', standard_name='latitude',
+    lat_coord = DimCoord([0.0, 10.0],
+                         var_name='lat',
+                         standard_name='latitude',
                          units='degrees')
     cubes = CubeList([
         Cube([1, 1], var_name='ta', units='K',
@@ -1111,25 +1267,24 @@ def test_only_latitude():
     np.testing.assert_allclose(new_lat_coord.points, [0.0, 10.0])
     assert new_lat_coord.bounds is None
 
-    # Restore original dimensions of ta
-    vardef.dimensions = original_dimensions
+    # Check that no mesh has been created
+    assert cube.mesh is None
 
 
-def test_only_longitude():
+def test_only_longitude(monkeypatch):
     """Test fix."""
+    fix = get_allvars_fix('Amon', 'ta')
     # We know that ta has dimensions time, plev19, latitude, longitude, but the
     # ICON CMORizer is designed to check for the presence of each dimension
     # individually. To test this, remove all but one dimension of ta to create
     # an artificial, but realistic test case.
-    vardef = get_var_info('ICON', 'Amon', 'ta')
-    original_dimensions = vardef.dimensions
-    vardef.dimensions = ['longitude']
-    extra_facets = get_extra_facets('ICON', 'ICON', 'Amon', 'ta', ())
-    fix = AllVars(vardef, extra_facets=extra_facets)
+    monkeypatch.setattr(fix.vardef, 'dimensions', ['longitude'])
 
     # Create cube with only a single dimension
-    lon_coord = DimCoord([0.0, 180.0], var_name='lon',
-                         standard_name='longitude', units='degrees')
+    lon_coord = DimCoord([0.0, 180.0],
+                         var_name='lon',
+                         standard_name='longitude',
+                         units='degrees')
     cubes = CubeList([
         Cube([1, 1], var_name='ta', units='K',
              dim_coords_and_dims=[(lon_coord, 0)]),
@@ -1155,8 +1310,8 @@ def test_only_longitude():
     np.testing.assert_allclose(new_lon_coord.points, [0.0, 180.0])
     assert new_lon_coord.bounds is None
 
-    # Restore original dimensions of ta
-    vardef.dimensions = original_dimensions
+    # Check that no mesh has been created
+    assert cube.mesh is None
 
 
 # Test variable not available in file
@@ -1181,3 +1336,100 @@ def test_invalid_time_units(cubes_2d):
     msg = "Expected time units"
     with pytest.raises(ValueError, match=msg):
         fix.fix_metadata(cubes_2d)
+
+
+# Test mesh creation raises warning because bounds do not match vertices
+
+
+@mock.patch('esmvalcore.cmor._fixes.icon._base_fixes.logger', autospec=True)
+def test_get_mesh_fail_invalid_clat_bounds(mock_logger, cubes_2d):
+    """Test fix."""
+    # Slightly modify latitude bounds from tas cube to make mesh creation fail
+    tas_cube = cubes_2d.extract_cube(NameConstraint(var_name='tas'))
+    lat_bnds = tas_cube.coord('latitude').bounds.copy()
+    lat_bnds[0, 0] = 40.0
+    tas_cube.coord('latitude').bounds = lat_bnds
+    cubes = CubeList([tas_cube])
+    fix = get_allvars_fix('Amon', 'tas')
+
+    fixed_cubes = fix.fix_metadata(cubes)
+    cube = check_tas_metadata(fixed_cubes)
+
+    assert cube.coord('latitude').bounds[0, 0] != 40.0
+    mock_logger.warning.assert_called_once_with(
+        "Latitude bounds of the face coordinate ('clat_vertices' in "
+        "the grid file) differ from the corresponding values "
+        "calculated from the connectivity ('vertex_of_cell') and the "
+        "node coordinate ('vlat'). Using bounds defined by "
+        "connectivity."
+    )
+
+
+@mock.patch('esmvalcore.cmor._fixes.icon._base_fixes.logger', autospec=True)
+def test_get_mesh_fail_invalid_clon_bounds(mock_logger, cubes_2d):
+    """Test fix."""
+    # Slightly modify longitude bounds from tas cube to make mesh creation fail
+    tas_cube = cubes_2d.extract_cube(NameConstraint(var_name='tas'))
+    lon_bnds = tas_cube.coord('longitude').bounds.copy()
+    lon_bnds[0, 1] = 40.0
+    tas_cube.coord('longitude').bounds = lon_bnds
+    cubes = CubeList([tas_cube])
+    fix = get_allvars_fix('Amon', 'tas')
+
+    fixed_cubes = fix.fix_metadata(cubes)
+    cube = check_tas_metadata(fixed_cubes)
+
+    assert cube.coord('longitude').bounds[0, 1] != 40.0
+    mock_logger.warning.assert_called_once_with(
+        "Longitude bounds of the face coordinate ('clon_vertices' in "
+        "the grid file) differ from the corresponding values "
+        "calculated from the connectivity ('vertex_of_cell') and the "
+        "node coordinate ('vlon'). Note that these values are allowed "
+        "to differ by 360° or at the poles of the grid. Using bounds "
+        "defined by connectivity."
+    )
+
+
+# Test _get_grid_url
+
+
+def test_get_grid_url():
+    """Test fix."""
+    cube = Cube(0, attributes={'grid_file_uri': TEST_GRID_FILE_URI})
+    fix = get_allvars_fix('Amon', 'tas')
+    (grid_url, grid_name) = fix._get_grid_url(cube)
+    assert grid_url == TEST_GRID_FILE_URI
+    assert grid_name == TEST_GRID_FILE_NAME
+
+
+def test_get_grid_url_fail():
+    """Test fix."""
+    cube = Cube(0)
+    fix = get_allvars_fix('Amon', 'tas')
+    msg = ("Cube does not contain the attribute 'grid_file_uri' necessary to "
+           "download the ICON horizontal grid file")
+    with pytest.raises(ValueError, match=msg):
+        fix._get_grid_url(cube)
+
+
+# Test get_mesh
+
+
+def test_get_mesh_cached(monkeypatch):
+    """Test fix."""
+    cube = Cube(0, attributes={'grid_file_uri': TEST_GRID_FILE_URI})
+    fix = get_allvars_fix('Amon', 'tas')
+    monkeypatch.setattr(fix, '_create_mesh', mock.Mock())
+    fix._meshes[TEST_GRID_FILE_NAME] = mock.sentinel.mesh
+    mesh = fix.get_mesh(cube)
+    assert mesh == mock.sentinel.mesh
+    fix._create_mesh.assert_not_called()
+
+
+def test_get_mesh_not_cached(monkeypatch):
+    """Test fix."""
+    cube = Cube(0, attributes={'grid_file_uri': TEST_GRID_FILE_URI})
+    fix = get_allvars_fix('Amon', 'tas')
+    monkeypatch.setattr(fix, '_create_mesh', mock.Mock())
+    fix.get_mesh(cube)
+    fix._create_mesh.assert_called_once_with(cube)
