@@ -3,11 +3,12 @@
 import os
 import tempfile
 import unittest
+import warnings
 
 import iris
 import numpy as np
 from iris.coords import DimCoord
-from iris.cube import Cube
+from iris.cube import Cube, CubeList
 
 from esmvalcore.preprocessor._io import concatenate_callback, load
 
@@ -20,6 +21,7 @@ def _create_sample_cube():
 
 class TestLoad(unittest.TestCase):
     """Tests for :func:`esmvalcore.preprocessor.load`."""
+
     def setUp(self):
         """Start tests."""
         self.temp_files = []
@@ -58,7 +60,7 @@ class TestLoad(unittest.TestCase):
                 cube.attributes[attr] = attr
             self._save_cube(cube)
         for temp_file in self.temp_files:
-            cubes = load(temp_file, callback=concatenate_callback)
+            cubes = load(temp_file, callback='default')
             cube = cubes[0]
             self.assertEqual(1, len(cubes))
             self.assertTrue((cube.data == np.array([1, 2])).all())
@@ -77,7 +79,7 @@ class TestLoad(unittest.TestCase):
                     coord.attributes[attr] = attr
             self._save_cube(cube)
         for temp_file in self.temp_files:
-            cubes = load(temp_file, callback=concatenate_callback)
+            cubes = load(temp_file, callback='default')
             cube = cubes[0]
             self.assertEqual(1, len(cubes))
             self.assertTrue((cube.data == np.array([1, 2])).all())
@@ -99,3 +101,47 @@ class TestLoad(unittest.TestCase):
         self.assertTrue((cube.coord('latitude').points == np.array([1,
                                                                     2])).all())
         self.assertEqual(cube.coord('latitude').units, 'degrees_north')
+
+    @unittest.mock.patch('iris.load_raw', autospec=True)
+    def test_fail_empty_cubes(self, mock_load_raw):
+        """Test that ValueError is raised when cubes are empty."""
+        mock_load_raw.return_value = CubeList([])
+        msg = "Can not load cubes from myfilename"
+        with self.assertRaises(ValueError, msg=msg):
+            load('myfilename')
+
+    @staticmethod
+    def load_with_warning(*_, **__):
+        """Mock load with a warning."""
+        warnings.warn("This is a custom expected warning",
+                      category=UserWarning)
+        return CubeList([Cube(0)])
+
+    @unittest.mock.patch('iris.load_raw', autospec=True)
+    def test_do_not_ignore_warnings(self, mock_load_raw):
+        """Test do not ignore specific warnings."""
+        mock_load_raw.side_effect = self.load_with_warning
+        ignore_warnings = [{'message': "non-relevant warning"}]
+
+        # Warning is not ignored -> assert warning has been issued
+        with self.assertWarns(UserWarning):
+            cubes = load('myfilename', ignore_warnings=ignore_warnings)
+
+        # Check output
+        self.assertEqual(len(cubes), 1)
+        self.assertEqual(cubes[0].attributes, {'source_file': 'myfilename'})
+
+    @unittest.mock.patch('iris.load_raw', autospec=True)
+    def test_ignore_warnings(self, mock_load_raw):
+        """Test ignore specific warnings."""
+        mock_load_raw.side_effect = self.load_with_warning
+        ignore_warnings = [{'message': "This is a custom expected warning"}]
+
+        # Warning is ignored -> assert warning has not been issued
+        with self.assertRaises(AssertionError):
+            with self.assertWarns(UserWarning):
+                cubes = load('myfilename', ignore_warnings=ignore_warnings)
+
+        # Check output
+        self.assertEqual(len(cubes), 1)
+        self.assertEqual(cubes[0].attributes, {'source_file': 'myfilename'})
