@@ -8,8 +8,9 @@ from pathlib import Path
 import iris
 import pytest
 
-from esmvalcore._config import TAGS
-from esmvalcore._recipe import RecipeError
+from esmvalcore.config._config_object import CFG_DEFAULT
+from esmvalcore.config._diagnostics import TAGS
+from esmvalcore.exceptions import RecipeError
 from esmvalcore.experimental import CFG, Recipe, get_recipe
 from esmvalcore.experimental.recipe_output import (
     DataFile,
@@ -19,8 +20,6 @@ from esmvalcore.experimental.recipe_output import (
 
 esmvaltool_sample_data = pytest.importorskip("esmvaltool_sample_data")
 
-CFG.update(esmvaltool_sample_data.get_rootpaths())
-CFG['max_parallel_tasks'] = 1
 
 AUTHOR_TAGS = {
     'authors': {
@@ -41,20 +40,25 @@ def recipe():
 
 @pytest.mark.use_sample_data
 @pytest.mark.parametrize('task', (None, 'example/ta'))
-def test_run_recipe(task, recipe, tmp_path):
+def test_run_recipe(monkeypatch, task, recipe, tmp_path):
     """Test running a basic recipe using sample data.
 
     Recipe contains no provenance and no diagnostics.
     """
     TAGS.set_tag_values(AUTHOR_TAGS)
 
-    CFG['output_dir'] = tmp_path
-
     assert isinstance(recipe, Recipe)
     assert isinstance(recipe._repr_html_(), str)
 
+    sample_data_config = esmvaltool_sample_data.get_rootpaths()
+    monkeypatch.setitem(CFG, 'rootpath', sample_data_config['rootpath'])
+    monkeypatch.setitem(CFG, 'drs', {'CMIP6': 'SYNDA'})
     session = CFG.start_session(recipe.path.stem)
-    session['extra_facets_dir'] = []
+    session.clear()
+    session.update(CFG_DEFAULT)
+    session['output_dir'] = tmp_path / 'esmvaltool_output'
+    session['max_parallel_tasks'] = 1
+    session['remove_preproc_dir'] = False
 
     output = recipe.run(task=task, session=session)
 
@@ -79,17 +83,16 @@ def test_run_recipe(task, recipe, tmp_path):
 
 
 @pytest.mark.use_sample_data
-def test_run_recipe_diagnostic_failing(recipe, tmp_path):
+def test_run_recipe_diagnostic_failing(monkeypatch, recipe, tmp_path):
     """Test running a single diagnostic using sample data.
 
     Recipe contains no provenance and no diagnostics.
     """
     TAGS.set_tag_values(AUTHOR_TAGS)
 
-    CFG['output_dir'] = tmp_path
+    monkeypatch.setitem(CFG, 'output_dir', tmp_path)
 
     session = CFG.start_session(recipe.path.stem)
-    session['extra_facets_dir'] = []
 
     with pytest.raises(RecipeError):
         task = 'example/non-existant'
