@@ -1,15 +1,23 @@
 """Unit tests for the CMORCheck class."""
 
 import unittest
+from copy import deepcopy
 
+import dask.array as da
 import iris
 import iris.coord_categorisation
 import iris.coords
+import iris.cube
 import iris.util
 import numpy as np
 from cf_units import Unit
 
-from esmvalcore.cmor.check import CMORCheck, CMORCheckError, CheckLevels
+from esmvalcore.cmor.check import (
+    CheckLevels,
+    CMORCheck,
+    CMORCheckError,
+    _get_cmor_checker,
+)
 
 
 class VariableInfoMock:
@@ -38,7 +46,7 @@ class VariableInfoMock:
             'lat': CoordinateInfoMock('lat'),
             'lon': CoordinateInfoMock('lon'),
             'air_pressure': requested,
-            'depth': generic_level
+            'depth': generic_level,
         }
 
 
@@ -277,37 +285,28 @@ class TestCMORCheck(unittest.TestCase):
 
     def test_rank_unstructured_grid(self):
         """Check succeeds even if two required coordinates share dimensions."""
-        self.cube = self.cube.extract(
-            iris.Constraint(latitude=self.cube.coord('latitude').points[0]))
-        lat_points = self.cube.coord('longitude').points
-        lat_points = lat_points / 3.0 - 50.
-        self.cube.remove_coord('latitude')
-        iris.util.demote_dim_coord_to_aux_coord(self.cube, 'longitude')
-        lat_points = np.concatenate(
-            (
-                self.cube.coord('longitude').points[0:10] / 4,
-                self.cube.coord('longitude').points[0:10] / 4
-            ),
-            axis=0
-        )
-        lat_bounds = np.concatenate(
-            (
-                self.cube.coord('longitude').bounds[0:10] / 4,
-                self.cube.coord('longitude').bounds[0:10] / 4
-            ),
-            axis=0
-        )
-        new_lat = iris.coords.AuxCoord(
-            points=lat_points,
-            bounds=lat_bounds,
-            var_name='lat',
-            standard_name='latitude',
-            long_name='Latitude',
-            units='degrees_north',
-        )
-        self.cube.add_aux_coord(new_lat, 1)
+        self.cube = self._get_unstructed_grid_cube()
         self._check_cube()
 
+    def test_unstructured_grid_automatic_fixes(self):
+        """Check succeeds even if two required coordinates share dimensions."""
+        self.cube = self._get_unstructed_grid_cube(correct_lon=False,
+                                                   n_bounds=3)
+
+        lon_coord = self.cube.coord('longitude')
+        np.testing.assert_allclose(lon_coord.points.min(), -180.0)
+        np.testing.assert_allclose(lon_coord.points.max(), 162.0)
+        np.testing.assert_allclose(lon_coord.bounds.min(), -180.0)
+        np.testing.assert_allclose(lon_coord.bounds.max(), 171.0)
+
+        self._check_cube(automatic_fixes=True)
+
+        lon_coord = self.cube.coord('longitude')
+        np.testing.assert_allclose(lon_coord.points.min(), 0.0)
+        np.testing.assert_allclose(lon_coord.points.max(), 342.0)
+        np.testing.assert_allclose(lon_coord.bounds.min(), 0.0)
+        np.testing.assert_allclose(lon_coord.bounds.max(), 351.0)
+
     def test_bad_generic_level(self):
         """Test check fails in metadata if generic level coord
         has wrong var_name."""
@@ -322,6 +321,57 @@ class TestCMORCheck(unittest.TestCase):
         self.var_info.coordinates['depth'].out_name = ""
         self._check_fails_in_metadata()
 
+    def test_valid_generic_level(self):
+        """Test valid generic level coordinate."""
+        self._setup_generic_level_var()
+        checker = CMORCheck(self.cube, self.var_info)
+        checker.check_metadata()
+        checker.check_data()
+
+    def test_invalid_generic_level(self):
+        """Test invalid generic level coordinate."""
+        self._setup_generic_level_var()
+        self.cube.remove_coord('atmosphere_sigma_coordinate')
+        self._check_fails_in_metadata()
+
+    def test_generic_level_alternative_cmip3(self):
+        """Test valid alternative for generic level coords (CMIP3)."""
+        self.var_info.table_type = 'CMIP3'
+        self._setup_generic_level_var()
+        self.var_info.coordinates['zlevel'] = self.var_info.coordinates.pop(
+            'alevel')
+        self._add_plev_to_cube()
+        self._check_warnings_on_metadata()
+
+    def test_generic_level_alternative_cmip5(self):
+        """Test valid alternative for generic level coords (CMIP5)."""
+        self.var_info.table_type = 'CMIP5'
+        self._setup_generic_level_var()
+        self._add_plev_to_cube()
+        self._check_warnings_on_metadata()
+
+    def test_generic_level_alternative_cmip6(self):
+        """Test valid alternative for generic level coords (CMIP6)."""
+        self.var_info.table_type = 'CMIP6'
+        self._setup_generic_level_var()
+        self._add_plev_to_cube()
+        self._check_warnings_on_metadata()
+
+    def test_generic_level_alternative_obs4mips(self):
+        """Test valid alternative for generic level coords (obs4MIPs)."""
+        self.var_info.table_type = 'obs4MIPs'
+        self._setup_generic_level_var()
+        self._add_plev_to_cube()
+        self._check_warnings_on_metadata()
+
+    def test_generic_level_invalid_alternative(self):
+        """Test invalid alternative for generic level coords."""
+        self.var_info.table_type = 'CMIP6'
+        self._setup_generic_level_var()
+        self._add_plev_to_cube()
+        self.cube.coord('air_pressure').standard_name = 'altitude'
+        self._check_fails_in_metadata()
+
     def test_check_bad_var_standard_name_strict_flag(self):
         """Test check fails for a bad variable standard_name with
         --cmor-check strict."""
@@ -549,6 +599,34 @@ class TestCMORCheck(unittest.TestCase):
         self._check_warnings_on_metadata(automatic_fixes=False,
                                          check_level=CheckLevels.IGNORE)
 
+    def test_check_lazy(self):
+        """Test checker does not realise data or aux_coords."""
+        self.cube.data = self.cube.lazy_data()
+        self.cube.remove_coord('latitude')
+        self.cube.remove_coord('longitude')
+        self.cube.add_aux_coord(
+            iris.coords.AuxCoord(
+                da.reshape(da.linspace(-90, 90, num=20*20), (20, 20)),
+                var_name='lat',
+                standard_name='latitude',
+                units='degrees_north'
+            ),
+            (1, 2)
+        )
+        self.cube.add_aux_coord(
+            iris.coords.AuxCoord(
+                da.reshape(da.linspace(0, 360, num=20*20), (20, 20)),
+                var_name='lon',
+                standard_name='longitude',
+                units='degrees_east'
+            ),
+            (1, 2)
+        )
+        self._check_cube()
+        self.assertTrue(self.cube.coord('latitude').has_lazy_points())
+        self.assertTrue(self.cube.coord('longitude').has_lazy_points())
+        self.assertTrue(self.cube.has_lazy_data())
+
     def _check_fails_in_metadata(self, automatic_fixes=False, frequency=None,
                                  check_level=CheckLevels.DEFAULT):
         checker = CMORCheck(
@@ -589,6 +667,20 @@ class TestCMORCheck(unittest.TestCase):
         checker.check_metadata()
         self.assertTrue(checker.has_warnings())
 
+    def test_almost_requested(self):
+        """Automatically fix tiny deviations from the requested values."""
+        coord = self.cube.coord('air_pressure')
+        values = np.array(coord.points)
+        values[0] += 1.e-7
+        values[-1] += 1.e-7
+        self._update_coordinate_values(self.cube, coord, values)
+        checker = CMORCheck(self.cube, self.var_info, automatic_fixes=True)
+        checker.check_metadata()
+        reference = np.array(
+            self.var_info.coordinates['air_pressure'].requested, dtype=float)
+        np.testing.assert_array_equal(
+            self.cube.coord('air_pressure').points, reference)
+
     def test_requested_str_values(self):
         """
         Warning if requested values are not present.
@@ -608,6 +700,23 @@ class TestCMORCheck(unittest.TestCase):
         self.cube = self.get_cube(self.var_info)
         self._check_cube()
 
+    def test_requested_non_1d(self):
+        """Warning if requested values in non-1d cannot be checked."""
+        coord = self.cube.coord('air_pressure')
+        values = np.linspace(0, 40, len(coord.points))
+        values = np.broadcast_to(values, (20, 20))
+        bounds = np.moveaxis(np.stack((values - 0.01, values + 0.01)), 0, -1)
+        new_plev_coord = iris.coords.AuxCoord(
+            values, bounds=bounds, var_name=coord.var_name,
+            standard_name=coord.standard_name, long_name=coord.long_name,
+            units=coord.units)
+        self.cube.remove_coord('air_pressure')
+        self.cube.add_aux_coord(new_plev_coord, (2, 3))
+        checker = CMORCheck(self.cube, self.var_info)
+        checker.check_metadata()
+        self.assertFalse(checker.has_debug_messages())
+        self.assertTrue(checker.has_warnings())
+
     def test_non_increasing(self):
         """Fail in metadata if increasing coordinate is decreasing."""
         coord = self.cube.coord('latitude')
@@ -654,8 +763,7 @@ class TestCMORCheck(unittest.TestCase):
         cube_points = self.cube.coord('latitude').points
         reference = np.linspace(90, -90, 20, endpoint=True)
         for index in range(20):
-            self.assertTrue(
-                iris.util.approx_equal(cube_points[index], reference[index]))
+            np.testing.assert_allclose(cube_points[index], reference[index])
         # test bounds are contiguous
         bounds = self.cube.coord('latitude').bounds
         right_bounds = bounds[:-2, 1]
@@ -877,6 +985,18 @@ class TestCMORCheck(unittest.TestCase):
         )
         self._check_debug_messages_on_metadata(automatic_fixes=True)
 
+    def test_bad_out_name_region_area_type(self):
+        """Debug message if region/area_type AuxCoord has bad var_name at
+        metadata."""
+        region_coord = CoordinateInfoMock('basin')
+        region_coord.standard_name = 'region'
+        self.var_info.coordinates['region'] = region_coord
+
+        self.cube = self.get_cube(self.var_info)
+        self.cube.coord("region").var_name = 'sector'
+
+        self._check_debug_messages_on_metadata()
+
     def test_bad_out_name_onedim_latitude(self):
         """Warning if onedimensional lat has bad var_name at metadata"""
         self.var_info.table_type = 'CMIP6'
@@ -989,6 +1109,27 @@ class TestCMORCheck(unittest.TestCase):
         guessed_bounds = self.cube.coord('time').bounds
         assert guessed_bounds is None
 
+    def test_hr_mip_cordex(self):
+        """Test hourly CORDEX tables are found."""
+        checker = _get_cmor_checker('CORDEX', '3hr', 'tas', '3hr')
+        assert checker(self.cube)._cmor_var.short_name == 'tas'
+        assert checker(self.cube)._cmor_var.frequency == '3hr'
+
+    def test_set_range_in_0_360(self):
+        checker = _get_cmor_checker('CMIP5', 'Amon', 'tas', 'mon')
+        arr_in = np.array([[-120.0, 0.0, 20.0], [-1.0, 360.0, 400.0]])
+        arr_exp = np.array([[240.0, 0.0, 20.0], [359.0, 0.0, 40.0]])
+        arr_out = checker(self.cube)._set_range_in_0_360(arr_in)
+        np.testing.assert_allclose(arr_out, arr_exp)
+
+    def test_set_range_in_0_360_lazy(self):
+        checker = _get_cmor_checker('CMIP5', 'Amon', 'tas', 'mon')
+        arr_in = da.from_array([[-120.0, 0.0, 20.0], [-1.0, 360.0, 400.0]])
+        arr_exp = da.from_array([[240.0, 0.0, 20.0], [359.0, 0.0, 40.0]])
+        arr_out = checker(self.cube)._set_range_in_0_360(arr_in)
+        self.assertTrue(isinstance(arr_out, da.core.Array))
+        np.testing.assert_allclose(arr_out.compute(), arr_exp.compute())
+
     def _check_fails_on_data(self):
         checker = CMORCheck(self.cube, self.var_info)
         checker.check_metadata()
@@ -1069,6 +1210,113 @@ class TestCMORCheck(unittest.TestCase):
 
         return cube
 
+    def _get_unstructed_grid_cube(self, correct_lon=True, n_bounds=2):
+        """Get cube with unstructured grid."""
+        assert n_bounds in (2, 3), "Only 2 or 3 bounds per cell supported"
+
+        cube = self.get_cube(self.var_info)
+        cube = cube.extract(
+            iris.Constraint(latitude=cube.coord('latitude').points[0]))
+        lat_points = cube.coord('longitude').points
+        lat_points = lat_points / 3.0 - 50.
+        cube.remove_coord('latitude')
+        iris.util.demote_dim_coord_to_aux_coord(cube, 'longitude')
+        lat_points = np.concatenate(
+            (
+                cube.coord('longitude').points[0:10] / 4,
+                cube.coord('longitude').points[0:10] / 4
+            ),
+            axis=0
+        )
+        lat_bounds = np.concatenate(
+            (
+                cube.coord('longitude').bounds[0:10] / 4,
+                cube.coord('longitude').bounds[0:10] / 4
+            ),
+            axis=0
+        )
+        new_lat = iris.coords.AuxCoord(
+            points=lat_points,
+            bounds=lat_bounds,
+            var_name='lat',
+            standard_name='latitude',
+            long_name='Latitude',
+            units='degrees_north',
+        )
+        cube.add_aux_coord(new_lat, 1)
+
+        # Add additional bound if desired
+        if n_bounds == 3:
+            for coord_name in ('latitude', 'longitude'):
+                coord = cube.coord(coord_name)
+                new_bounds = np.stack((
+                    coord.bounds[:, 0],
+                    0.5 * (coord.bounds[:, 0] + coord.bounds[:, 1]),
+                    coord.bounds[:, 1],
+                ))
+                coord.bounds = np.swapaxes(new_bounds, 0, 1)
+
+        # Convert longitude to [-180, 180] to check automatic fixes if desired
+        if correct_lon:
+            return cube
+        new_lon = iris.coords.AuxCoord(
+            points=cube.coord('longitude').points,
+            bounds=cube.coord('longitude').bounds,
+            var_name='lon',
+            standard_name='longitude',
+            long_name='Longitude',
+            units='degrees_east',
+        )
+        new_lon.points = np.where(new_lon.points < 180.0, new_lon.points,
+                                  new_lon.points - 360.0)
+        new_lon.bounds = np.where(new_lon.bounds < 180.0, new_lon.bounds,
+                                  new_lon.bounds - 360.0)
+        cube.remove_coord('longitude')
+        cube.add_aux_coord(new_lon, 1)
+        return cube
+
+    def _setup_generic_level_var(self):
+        """Setup var_info and cube with generic alevel coordinate."""
+        self.var_info.coordinates.pop('depth')
+        self.var_info.coordinates.pop('air_pressure')
+
+        # Create cube with sigma coordinate
+        sigma_coord = CoordinateInfoMock('standard_sigma')
+        sigma_coord.axis = 'Z'
+        sigma_coord.out_name = 'lev'
+        sigma_coord.standard_name = 'atmosphere_sigma_coordinate'
+        sigma_coord.long_name = 'sigma coordinate'
+        sigma_coord.generic_lev_name = 'alevel'
+        var_info_for_cube = deepcopy(self.var_info)
+        var_info_for_cube.coordinates['standard_sigma'] = sigma_coord
+        self.cube = self.get_cube(var_info_for_cube)
+
+        # Create var_info with alevel coord that contains sigma coordinate in
+        # generic_lev_coords dict (just like it is the case for the true CMOR
+        # tables)
+        gen_lev_coord = CoordinateInfoMock('alevel')
+        gen_lev_coord.standard_name = None
+        gen_lev_coord.generic_level = True
+        gen_lev_coord.generic_lev_coords = {'standard_sigma': sigma_coord}
+        self.var_info.coordinates['alevel'] = gen_lev_coord
+
+    def _add_plev_to_cube(self):
+        """Add plev coordinate to cube."""
+        if self.cube.coords('atmosphere_sigma_coordinate'):
+            self.cube.remove_coord('atmosphere_sigma_coordinate')
+        plevs = [100000.0, 92500.0, 85000.0, 70000.0, 60000.0, 50000.0,
+                 40000.0, 30000.0, 25000.0, 20000.0, 15000.0, 10000.0, 7000.0,
+                 5000.0, 3000.0, 2000.0, 1000.0, 900.0, 800.0, 700.0]
+        coord = iris.coords.DimCoord(
+            plevs,
+            var_name='plev',
+            standard_name='air_pressure',
+            units='Pa',
+            attributes={'positive': 'down'},
+        )
+        coord.guess_bounds()
+        self.cube.add_dim_coord(coord, 3)
+
     def _get_valid_limits(self, var_info):
         if var_info.valid_min:
             valid_min = float(var_info.valid_min)
