@@ -1,5 +1,6 @@
 import pytest
 
+from esmvalcore.config._diagnostics import DIAGNOSTICS, TAGS
 from esmvalcore.experimental.recipe import Recipe
 from esmvalcore.experimental.utils import (
     RecipeList,
@@ -9,7 +10,7 @@ from esmvalcore.experimental.utils import (
 
 pytest.importorskip(
     'esmvaltool',
-    reason='The behaviour of these tests depends on what ``DIAGNOSTICS_PATH``'
+    reason='The behaviour of these tests depends on what ``DIAGNOSTICS.path``'
     'points to. This is defined by a forward-reference to ESMValTool, which'
     'is not installed in the CI, but likely to be available in a developer'
     'or user installation.')
@@ -29,6 +30,8 @@ def test_get_all_recipes():
 
 def test_recipe_list_find():
     """Get all recipes."""
+    TAGS.set_tag_values(DIAGNOSTICS.load_tags())
+
     recipes = get_all_recipes(subdir='examples')
 
     assert isinstance(recipes, RecipeList)
