@@ -1,14 +1,16 @@
 """Unit tests for the :func:`esmvalcore.preprocessor._time` module."""
 
 import copy
-import datetime
+import re
 import unittest
+from datetime import datetime
 from typing import List, Tuple
 
 import iris
 import iris.coord_categorisation
 import iris.coords
 import iris.exceptions
+import iris.fileformats
 import numpy as np
 import pytest
 from cf_units import Unit
@@ -20,11 +22,12 @@ from numpy.testing import (
 )
 
 import tests
+from esmvalcore.iris_helpers import date2num
 from esmvalcore.preprocessor._time import (
     annual_statistics,
     anomalies,
     climate_statistics,
-    clip_start_end_year,
+    clip_timerange,
     daily_statistics,
     decadal_statistics,
     extract_month,
@@ -42,6 +45,7 @@ from esmvalcore.preprocessor._time import (
 
 
 def _create_sample_cube(calendar='gregorian'):
+    """Create sample cube."""
     cube = Cube(np.arange(1, 25), var_name='co2', units='J')
     cube.add_dim_coord(
         iris.coords.DimCoord(
@@ -144,6 +148,7 @@ class TestTimeSlice(tests.Test):
 
     def test_extract_time_no_slice(self):
         """Test fail of extract_time."""
+        self.cube.coord('time').guess_bounds()
         with self.assertRaises(ValueError) as ctx:
             extract_time(self.cube, 2200, 1, 1, 2200, 12, 31)
         msg = ("Time slice 2200-01-01 to 2200-12-31 is outside"
@@ -165,47 +170,273 @@ class TestTimeSlice(tests.Test):
         assert cube == sliced
 
 
-class TestClipStartEndYear(tests.Test):
-    """Tests for clip_start_end_year."""
+class TestClipTimerange(tests.Test):
+    """Tests for clip_timerange."""
     def setUp(self):
         """Prepare tests."""
         self.cube = _create_sample_cube()
 
-    def test_clip_start_end_year_1_year(self):
-        """Test clip_start_end_year with 1 year."""
-        sliced = clip_start_end_year(self.cube, 1950, 1950)
+    @staticmethod
+    def _create_cube(data, times, bounds, calendar='gregorian'):
+        time = iris.coords.DimCoord(times,
+                                    bounds=bounds,
+                                    standard_name='time',
+                                    units=Unit('days since 1950-01-01',
+                                               calendar=calendar))
+        cube = iris.cube.Cube(data, dim_coords_and_dims=[(time, 0)])
+        return cube
+
+    def test_clip_timerange_1_year(self):
+        """Test clip_timerange with 1 year."""
+        sliced = clip_timerange(self.cube, '1950/1950')
         iris.coord_categorisation.add_month_number(sliced, 'time')
         iris.coord_categorisation.add_year(sliced, 'time')
         assert_array_equal(np.arange(1, 13, 1),
                            sliced.coord('month_number').points)
         assert_array_equal(np.full(12, 1950), sliced.coord('year').points)
 
-    def test_clip_start_end_year_3_years(self):
-        """Test clip_start_end_year with 3 years."""
-        sliced = clip_start_end_year(self.cube, 1949, 1951)
+    def test_clip_timerange_3_years(self):
+        """Test clip_timerange with 3 years."""
+        sliced = clip_timerange(self.cube, '1949/1951')
         assert sliced == self.cube
 
-    def test_clip_start_end_year_no_slice(self):
-        """Test fail of clip_start_end_year."""
-        with self.assertRaises(ValueError) as ctx:
-            clip_start_end_year(self.cube, 2200, 2200)
-        msg = ("Time slice 2200-01-01 to 2201-01-01 is outside"
+    def test_clip_timerange_no_slice(self):
+        """Test fail of clip_timerange."""
+        self.cube.coord('time').guess_bounds()
+        msg = ("Time slice 2200-01-01 01:00:00 to 2201-01-01 is outside"
                " cube time bounds 1950-01-16 00:00:00 to 1951-12-07 00:00:00.")
+        with self.assertRaises(ValueError) as ctx:
+            clip_timerange(self.cube, '22000101T010000/2200')
         assert ctx.exception.args == (msg, )
 
-    def test_clip_start_end_year_one_time(self):
-        """Test clip_start_end_year with one time step."""
+    def test_clip_timerange_one_time(self):
+        """Test clip_timerange with one time step."""
         cube = _create_sample_cube()
-        cube.coord('time').guess_bounds()
         cube = cube.collapsed('time', iris.analysis.MEAN)
-        sliced = clip_start_end_year(cube, 1950, 1952)
+        sliced = clip_timerange(cube, '1950/1952')
         assert_array_equal(np.array([360.]), sliced.coord('time').points)
 
-    def test_clip_start_end_year_no_time(self):
-        """Test clip_start_end_year with no time step."""
+    def test_clip_timerange_no_time(self):
+        """Test clip_timerange with no time step."""
         cube = _create_sample_cube()[0]
-        sliced = clip_start_end_year(cube, 1950, 1950)
-        assert cube == sliced
+        sliced_timerange = clip_timerange(cube, '1950/1950')
+        assert cube == sliced_timerange
+
+    def test_clip_timerange_date(self):
+        """Test timerange with dates."""
+        sliced_year = clip_timerange(self.cube, '1950/1952')
+        sliced_month = clip_timerange(self.cube, '195001/195212')
+        sliced_day = clip_timerange(self.cube, '19500101/19521231')
+        assert self.cube == sliced_year
+        assert self.cube == sliced_month
+        assert self.cube == sliced_day
+
+    def test_clip_timerange_datetime(self):
+        """Test timerange with datetime periods."""
+        data = np.arange(8)
+        times = np.arange(0, 48, 6)
+        time = iris.coords.DimCoord(times,
+                                    standard_name='time',
+                                    units=Unit('hours since 1950-01-01',
+                                               calendar='360_day'))
+        time.guess_bounds()
+        cube = iris.cube.Cube(data, dim_coords_and_dims=[(time, 0)])
+
+        sliced_cube = clip_timerange(cube, '19500101T000000/19500101T120000')
+        expected_time = np.arange(0, 18, 6)
+        assert_array_equal(sliced_cube.coord(time).points, expected_time)
+
+    def test_clip_timerange_monthly(self):
+        """Test timerange with monthly data."""
+        time = np.arange(15., 2175., 30)
+        data = np.ones_like(time)
+        calendars = [
+            '360_day', '365_day', '366_day',
+            'gregorian', 'julian', 'proleptic_gregorian']
+        for calendar in calendars:
+            cube = self._create_cube(data, time, None, calendar)
+            sliced_forward = clip_timerange(cube, '195001/P4Y6M')
+            sliced_backward = clip_timerange(cube, 'P4Y6M/195406')
+            assert sliced_forward.coord('time').cell(0).point.year == 1950
+            assert sliced_forward.coord('time').cell(-1).point.year == 1954
+            assert sliced_forward.coord('time').cell(0).point.month == 1
+            assert sliced_forward.coord('time').cell(-1).point.month == 6
+
+            assert sliced_backward.coord('time').cell(-1).point.year == 1954
+            assert sliced_backward.coord('time').cell(0).point.year == 1950
+            assert sliced_backward.coord('time').cell(-1).point.month == 6
+            assert sliced_backward.coord('time').cell(0).point.month == 1
+
+    def test_clip_timerange_daily(self):
+        """Test timerange with daily data."""
+        time = np.arange(0., 3000.)
+        data = np.ones_like(time)
+        calendars = [
+            '360_day', '365_day', '366_day',
+            'gregorian', 'julian', 'proleptic_gregorian']
+        for calendar in calendars:
+            cube = self._create_cube(data, time, None, calendar)
+            sliced_forward = clip_timerange(cube, '19500101/P4Y6M2D')
+            sliced_backward = clip_timerange(cube, 'P4Y6M3D/19540703')
+            assert sliced_forward.coord('time').cell(0).point.year == 1950
+            assert sliced_forward.coord('time').cell(-1).point.year == 1954
+            assert sliced_forward.coord('time').cell(0).point.month == 1
+            assert sliced_forward.coord('time').cell(-1).point.month == 7
+            assert sliced_forward.coord('time').cell(0).point.day == 1
+            assert sliced_forward.coord('time').cell(-1).point.day == 2
+
+            assert sliced_backward.coord('time').cell(-1).point.year == 1954
+            assert sliced_backward.coord('time').cell(0).point.year == 1950
+            assert sliced_backward.coord('time').cell(-1).point.month == 7
+            assert sliced_backward.coord('time').cell(0).point.month == 1
+            assert sliced_backward.coord('time').cell(-1).point.day == 3
+            assert sliced_backward.coord('time').cell(0).point.day == 1
+
+    def test_clip_timerange_duration_seconds(self):
+        """Test timerange with duration periods with resolution up to
+        seconds."""
+        data = np.arange(8)
+        times = np.arange(0, 48, 6)
+        calendars = [
+            '360_day', '365_day', '366_day',
+            'gregorian', 'julian', 'proleptic_gregorian']
+        for calendar in calendars:
+            time = iris.coords.DimCoord(times,
+                                        standard_name='time',
+                                        units=Unit('hours since 1950-01-01',
+                                                   calendar=calendar))
+            time.guess_bounds()
+            cube = iris.cube.Cube(data, dim_coords_and_dims=[(time, 0)])
+            sliced_cube_start = clip_timerange(cube, 'PT12H/19500101T120000')
+            sliced_cube_end = clip_timerange(cube, '19500101T000000/PT12H')
+            expected_time = np.arange(0, 18, 6)
+            assert_array_equal(
+                sliced_cube_start.coord('time').points, expected_time)
+            assert_array_equal(
+                sliced_cube_end.coord('time').points, expected_time)
+
+    def test_clip_timerange_30_day(self):
+        """Test day 31 is converted to day 30 in 360_day calendars."""
+        time = np.arange(0., 3000.)
+        data = np.ones_like(time)
+        cube = self._create_cube(data, time, None, '360_day')
+        sliced_cube = clip_timerange(cube, '19500131/19500331')
+        expected_time = np.arange(29, 90, 1)
+        assert_array_equal(
+                sliced_cube.coord('time').points, expected_time)
+
+    def test_clip_timerange_single_year_1d(self):
+        """Test that single year stays dimensional coordinate."""
+        cube = self._create_cube([0.0], [150.0], [[0.0, 365.0]], 'standard')
+        sliced_cube = clip_timerange(cube, '1950/1950')
+
+        assert sliced_cube.coord('time').units == Unit(
+            'days since 1950-01-01', calendar='standard')
+        assert_array_equal(sliced_cube.coord('time').points, [150.0])
+        assert_array_equal(sliced_cube.coord('time').bounds, [[0.0, 365.0]])
+        assert cube.shape == sliced_cube.shape
+        assert sliced_cube.coord('time', dim_coords=True)
+
+        # Repeat test without bounds
+        cube.coord('time').bounds = None
+        sliced_cube = clip_timerange(cube, '1950/1950')
+
+        assert sliced_cube.coord('time').units == Unit(
+            'days since 1950-01-01', calendar='standard')
+        assert_array_equal(sliced_cube.coord('time').points, [150.0])
+        assert sliced_cube.coord('time').bounds is None
+        assert cube.shape == sliced_cube.shape
+        assert sliced_cube.coord('time', dim_coords=True)
+
+    def test_clip_timerange_single_year_2d(self):
+        """Test that single year stays dimensional coordinate."""
+        cube = self._create_cube([[0.0, 1.0]], [150.0], [[0.0, 365.0]],
+                                 'standard')
+        lat_coord = iris.coords.DimCoord([10.0, 20.0],
+                                         standard_name='latitude')
+        cube.add_dim_coord(lat_coord, 1)
+        sliced_cube = clip_timerange(cube, '1950/1950')
+
+        assert sliced_cube.coord('time').units == Unit(
+            'days since 1950-01-01', calendar='standard')
+        assert_array_equal(sliced_cube.coord('time').points, [150.0])
+        assert_array_equal(sliced_cube.coord('time').bounds, [[0.0, 365.0]])
+        assert cube.shape == sliced_cube.shape
+        assert sliced_cube.coord('time', dim_coords=True)
+
+        # Repeat test without bounds
+        cube.coord('time').bounds = None
+        sliced_cube = clip_timerange(cube, '1950/1950')
+
+        assert sliced_cube.coord('time').units == Unit(
+            'days since 1950-01-01', calendar='standard')
+        assert_array_equal(sliced_cube.coord('time').points, [150.0])
+        assert sliced_cube.coord('time').bounds is None
+        assert cube.shape == sliced_cube.shape
+        assert sliced_cube.coord('time', dim_coords=True)
+
+    def test_clip_timerange_single_year_4d(self):
+        """Test time is not scalar even when time is not first coordinate."""
+        cube = self._create_cube([[[[0.0, 1.0]]]], [150.0], [[0.0, 365.0]],
+                                 'standard')
+        plev_coord = iris.coords.DimCoord([1013.0],
+                                          standard_name='air_pressure')
+        lat_coord = iris.coords.DimCoord([10.0], standard_name='latitude')
+        lon_coord = iris.coords.DimCoord([0.0, 1.0], standard_name='longitude')
+        cube.add_dim_coord(plev_coord, 1)
+        cube.add_dim_coord(lat_coord, 2)
+        cube.add_dim_coord(lon_coord, 3)
+
+        # Order: plev, time, lat, lon
+        cube_1 = cube.copy()
+        cube_1.transpose([1, 0, 2, 3])
+        assert cube_1.shape == (1, 1, 1, 2)
+        sliced_cube = clip_timerange(cube_1, '1950/1950')
+
+        assert sliced_cube is not cube_1
+        assert sliced_cube.coord('time').units == Unit(
+            'days since 1950-01-01', calendar='standard')
+        assert_array_equal(sliced_cube.coord('time').points, [150.0])
+        assert_array_equal(sliced_cube.coord('time').bounds, [[0.0, 365.0]])
+        assert cube_1.shape == sliced_cube.shape
+        assert sliced_cube.coord('time', dim_coords=True)
+        for coord_name in [c.name() for c in cube_1.coords()]:
+            assert (sliced_cube.coord_dims(coord_name) ==
+                    cube_1.coord_dims(coord_name))
+
+        # Order: lat, lon, time, plev
+        cube_2 = cube.copy()
+        cube_2.transpose([2, 3, 0, 1])
+        assert cube_2.shape == (1, 2, 1, 1)
+        sliced_cube = clip_timerange(cube_2, '1950/1950')
+
+        assert sliced_cube is not cube_2
+        assert sliced_cube.coord('time').units == Unit(
+            'days since 1950-01-01', calendar='standard')
+        assert_array_equal(sliced_cube.coord('time').points, [150.0])
+        assert_array_equal(sliced_cube.coord('time').bounds, [[0.0, 365.0]])
+        assert cube_2.shape == sliced_cube.shape
+        assert sliced_cube.coord('time', dim_coords=True)
+        for coord_name in [c.name() for c in cube_2.coords()]:
+            assert (sliced_cube.coord_dims(coord_name) ==
+                    cube_2.coord_dims(coord_name))
+
+        # Order: lon, lat, plev, time
+        cube_3 = cube.copy()
+        cube_3.transpose([3, 2, 1, 0])
+        assert cube_3.shape == (2, 1, 1, 1)
+        sliced_cube = clip_timerange(cube_3, '1950/1950')
+
+        assert sliced_cube is not cube_3
+        assert sliced_cube.coord('time').units == Unit(
+            'days since 1950-01-01', calendar='standard')
+        assert_array_equal(sliced_cube.coord('time').points, [150.0])
+        assert_array_equal(sliced_cube.coord('time').bounds, [[0.0, 365.0]])
+        assert cube_3.shape == sliced_cube.shape
+        assert sliced_cube.coord('time', dim_coords=True)
+        for coord_name in [c.name() for c in cube_3.coords()]:
+            assert (sliced_cube.coord_dims(coord_name) ==
+                    cube_3.coord_dims(coord_name))
 
 
 class TestExtractSeason(tests.Test):
@@ -293,7 +524,7 @@ class TestExtractSeason(tests.Test):
 
 
 class TestClimatology(tests.Test):
-    """Test class for :func:`esmvalcore.preprocessor._time.climatology`"""
+    """Test class for :func:`esmvalcore.preprocessor._time.climatology`."""
     @staticmethod
     def _create_cube(data, times, bounds):
         time = iris.coords.DimCoord(times,
@@ -490,9 +721,38 @@ class TestClimatology(tests.Test):
         expected = np.array([(5 / 3)**0.5], dtype=np.float32)
         assert_array_equal(result.data, expected)
 
+    def test_time_dependent_fx(self):
+        """Test average time dimension in time-dependent fx vars."""
+        data = np.ones((3, 3, 3))
+        times = np.array([15., 45., 75.])
+        bounds = np.array([[0., 30.], [30., 60.], [60., 90.]])
+        cube = self._create_cube(data, times, bounds)
+        measure = iris.coords.CellMeasure(data,
+                                          standard_name='ocean_volume',
+                                          var_name='volcello',
+                                          units='m3',
+                                          measure='volume')
+        ancillary_var = iris.coords.AncillaryVariable(
+            data,
+            standard_name='land_ice_area_fraction',
+            var_name='sftgif',
+            units='%')
+        cube.add_cell_measure(measure, (0, 1, 2))
+        cube.add_ancillary_variable(ancillary_var, (0, 1, 2))
+        with self.assertLogs(level='DEBUG') as cm:
+            result = climate_statistics(cube, operator='mean', period='mon')
+        self.assertEqual(cm.records[0].getMessage(),
+                         'Averaging time dimension in measure volcello.')
+        self.assertEqual(
+            cm.records[1].getMessage(),
+            'Averaging time dimension in ancillary variable sftgif.')
+        self.assertEqual(result.cell_measure('ocean_volume').ndim, 2)
+        self.assertEqual(
+            result.ancillary_variable('land_ice_area_fraction').ndim, 2)
+
 
 class TestSeasonalStatistics(tests.Test):
-    """Test :func:`esmvalcore.preprocessor._time.seasonal_statistics`"""
+    """Test :func:`esmvalcore.preprocessor._time.seasonal_statistics`."""
     @staticmethod
     def _create_cube(data, times):
         time = iris.coords.DimCoord(times,
@@ -577,9 +837,55 @@ class TestSeasonalStatistics(tests.Test):
         expected = np.array([1])
         assert_array_equal(result.data, expected)
 
+    def test_time_dependent_fx(self):
+        """Test average time dimension in time-dependent fx vars."""
+        data = np.ones((12, 3, 3))
+        times = np.arange(15, 360, 30)
+        cube = self._create_cube(data, times)
+        measure = iris.coords.CellMeasure(data,
+                                          standard_name='ocean_volume',
+                                          var_name='volcello',
+                                          units='m3',
+                                          measure='volume')
+        ancillary_var = iris.coords.AncillaryVariable(
+            data,
+            standard_name='land_ice_area_fraction',
+            var_name='sftgif',
+            units='%')
+        cube.add_cell_measure(measure, (0, 1, 2))
+        cube.add_ancillary_variable(ancillary_var, (0, 1, 2))
+        with self.assertLogs(level='DEBUG') as cm:
+            result = seasonal_statistics(cube, operator='mean')
+        self.assertEqual(cm.records[0].getMessage(),
+                         'Averaging time dimension in measure volcello.')
+        self.assertEqual(
+            cm.records[1].getMessage(),
+            'Averaging time dimension in ancillary variable sftgif.')
+        self.assertEqual(result.cell_measure('ocean_volume').ndim, 2)
+        self.assertEqual(
+            result.ancillary_variable('land_ice_area_fraction').ndim, 2)
+
+    def test_season_not_available(self):
+        """Test that an exception is raised if a season is not available."""
+        data = np.arange(12)
+        times = np.arange(15, 360, 30)
+        cube = self._create_cube(data, times)
+        iris.coord_categorisation.add_season(
+            cube,
+            'time',
+            name='clim_season',
+            seasons=['JFMAMJ', 'JASOND'],
+        )
+        msg = (
+            "Seasons ('DJF', 'MAM', 'JJA', 'SON') do not match prior season "
+            "extraction ['JASOND', 'JFMAMJ']."
+        )
+        with pytest.raises(ValueError, match=re.escape(msg)):
+            seasonal_statistics(cube, 'mean')
+
 
 class TestMonthlyStatistics(tests.Test):
-    """Test :func:`esmvalcore.preprocessor._time.monthly_statistics`"""
+    """Test :func:`esmvalcore.preprocessor._time.monthly_statistics`."""
     @staticmethod
     def _create_cube(data, times):
         time = iris.coords.DimCoord(times,
@@ -644,9 +950,37 @@ class TestMonthlyStatistics(tests.Test):
         expected = np.array([1, 5, 9, 13, 17, 21, 25, 29, 33, 37, 41, 45])
         assert_array_equal(result.data, expected)
 
+    def test_time_dependent_fx(self):
+        """Test average time dimension in time-dependent fx vars."""
+        data = np.ones((3, 3, 3))
+        times = np.array([15., 45., 75.])
+        cube = self._create_cube(data, times)
+        measure = iris.coords.CellMeasure(data,
+                                          standard_name='ocean_volume',
+                                          var_name='volcello',
+                                          units='m3',
+                                          measure='volume')
+        ancillary_var = iris.coords.AncillaryVariable(
+            data,
+            standard_name='land_ice_area_fraction',
+            var_name='sftgif',
+            units='%')
+        cube.add_cell_measure(measure, (0, 1, 2))
+        cube.add_ancillary_variable(ancillary_var, (0, 1, 2))
+        with self.assertLogs(level='DEBUG') as cm:
+            result = monthly_statistics(cube, operator='mean')
+        self.assertEqual(cm.records[0].getMessage(),
+                         'Averaging time dimension in measure volcello.')
+        self.assertEqual(
+            cm.records[1].getMessage(),
+            'Averaging time dimension in ancillary variable sftgif.')
+        self.assertEqual(result.cell_measure('ocean_volume').ndim, 2)
+        self.assertEqual(
+            result.ancillary_variable('land_ice_area_fraction').ndim, 2)
+
 
 class TestHourlyStatistics(tests.Test):
-    """Test :func:`esmvalcore.preprocessor._time.hourly_statistics`"""
+    """Test :func:`esmvalcore.preprocessor._time.hourly_statistics`."""
     @staticmethod
     def _create_cube(data, times):
         time = iris.coords.DimCoord(times,
@@ -709,7 +1043,7 @@ class TestHourlyStatistics(tests.Test):
 
 
 class TestDailyStatistics(tests.Test):
-    """Test :func:`esmvalcore.preprocessor._time.monthly_statistics`"""
+    """Test :func:`esmvalcore.preprocessor._time.monthly_statistics`."""
     @staticmethod
     def _create_cube(data, times):
         time = iris.coords.DimCoord(times,
@@ -819,10 +1153,10 @@ class TestRegridTimeYearly(tests.Test):
         timeunit_1 = newcube_1.coord('time').units
         for i, time in enumerate(newcube_1.coord('time').points):
             year_1 = timeunit_1.num2date(time).year
-            expected_minbound = timeunit_1.date2num(
-                datetime.datetime(year_1, 1, 1))
-            expected_maxbound = timeunit_1.date2num(
-                datetime.datetime(year_1 + 1, 1, 1))
+            expected_minbound = date2num(datetime(year_1, 1, 1),
+                                         timeunit_1)
+            expected_maxbound = date2num(datetime(year_1 + 1, 1, 1),
+                                         timeunit_1)
             assert_array_equal(
                 newcube_1.coord('time').bounds[i],
                 np.array([expected_minbound, expected_maxbound]))
@@ -873,10 +1207,10 @@ class TestRegridTimeMonthly(tests.Test):
             if month_1 == 12:
                 next_month = 1
                 next_year += 1
-            expected_minbound = timeunit_1.date2num(
-                datetime.datetime(year_1, month_1, 1))
-            expected_maxbound = timeunit_1.date2num(
-                datetime.datetime(next_year, next_month, 1))
+            expected_minbound = date2num(datetime(year_1, month_1, 1),
+                                         timeunit_1)
+            expected_maxbound = date2num(datetime(next_year, next_month, 1),
+                                         timeunit_1)
             assert_array_equal(
                 newcube_1.coord('time').bounds[i],
                 np.array([expected_minbound, expected_maxbound]))
@@ -1131,7 +1465,6 @@ class TestTimeseriesFilter(tests.Test):
 
     def test_timeseries_filter_timecoord(self):
         """Test missing time axis."""
-        import iris.exceptions
         new_cube = self.cube.copy()
         new_cube.remove_coord(new_cube.coord('time'))
         with self.assertRaises(iris.exceptions.CoordinateNotFoundError):
@@ -1202,7 +1535,7 @@ def test_decadal_average(existing_coord):
     if existing_coord:
 
         def get_decade(coord, value):
-            """Callback function to get decades from cube."""
+            """Get decades from cube."""
             date = coord.units.num2date(value)
             return date.year - date.year % 10
 
@@ -1216,6 +1549,36 @@ def test_decadal_average(existing_coord):
     assert_array_equal(result.coord('time').points, expected_time)
 
 
+@pytest.mark.parametrize('existing_coord', [True, False])
+def test_decadal_average_time_dependent_fx(existing_coord):
+    """Test for decadal average."""
+    cube = make_time_series(number_years=20)
+    measure = iris.coords.CellMeasure(cube.data,
+                                      standard_name='ocean_volume',
+                                      var_name='volcello',
+                                      units='m3',
+                                      measure='volume')
+    ancillary_var = iris.coords.AncillaryVariable(
+        cube.data,
+        standard_name='land_ice_area_fraction',
+        var_name='sftgif',
+        units='%')
+    cube.add_cell_measure(measure, 0)
+    cube.add_ancillary_variable(ancillary_var, 0)
+    if existing_coord:
+        def get_decade(coord, value):
+            """Get decades from cube."""
+            date = coord.units.num2date(value)
+            return date.year - date.year % 10
+
+        iris.coord_categorisation.add_categorised_coord(
+            cube, 'decade', 'time', get_decade)
+    result = decadal_statistics(cube)
+    assert result.cell_measure('ocean_volume').data.shape == (1,)
+    assert result.ancillary_variable(
+        'land_ice_area_fraction').data.shape == (1,)
+
+
 @pytest.mark.parametrize('existing_coord', [True, False])
 def test_decadal_sum(existing_coord):
     """Test for decadal average."""
@@ -1223,7 +1586,7 @@ def test_decadal_sum(existing_coord):
     if existing_coord:
 
         def get_decade(coord, value):
-            """Callback function to get decades from cube."""
+            """Get decades from cube."""
             date = coord.units.num2date(value)
             return date.year - date.year % 10
 
@@ -1287,6 +1650,7 @@ for period in ('full', 'day', 'month', 'season'):
 
 @pytest.mark.parametrize('period', ['full'])
 def test_standardized_anomalies(period, standardize=True):
+    """Test standardized ``anomalies``."""
     cube = make_map_data(number_years=2)
     result = anomalies(cube, period, standardize=standardize)
     if period == 'full':
@@ -1307,6 +1671,7 @@ def test_standardized_anomalies(period, standardize=True):
 
 @pytest.mark.parametrize('period, reference', PARAMETERS)
 def test_anomalies_preserve_metadata(period, reference, standardize=False):
+    """Test that ``anomalies`` preserves metadata."""
     cube = make_map_data(number_years=2)
     cube.var_name = "si"
     cube.units = "m"
@@ -1321,6 +1686,7 @@ def test_anomalies_preserve_metadata(period, reference, standardize=False):
 
 @pytest.mark.parametrize('period, reference', PARAMETERS)
 def test_anomalies(period, reference, standardize=False):
+    """Test ``anomalies``."""
     cube = make_map_data(number_years=2)
     result = anomalies(cube, period, reference, standardize=standardize)
     if reference is None:
@@ -1372,6 +1738,7 @@ def test_anomalies(period, reference, standardize=False):
 
 
 def test_anomalies_custom_season():
+    """Test ``anomalies`` with custom season."""
     cube = make_map_data(number_years=2)
     result = anomalies(cube, 'season', seasons=('jfmamj', 'jasond'))
     anom = np.concatenate((
@@ -1404,6 +1771,16 @@ def get_1d_time():
     return time
 
 
+def get_2d_time():
+    """Get 2D time coordinate."""
+    time = iris.coords.AuxCoord([[20., 45.]],
+                                standard_name='time',
+                                bounds=[[[15., 30.], [30., 60.]]],
+                                units=Unit('days since 1950-01-01',
+                                           calendar='gregorian'))
+    return time
+
+
 def get_lon_coord():
     """Get longitude coordinate."""
     lons = iris.coords.DimCoord([1.5, 2.5, 3.5],
@@ -1442,9 +1819,8 @@ def test_get_time_weights():
     """Test ``get_time_weights`` for complex cube."""
     cube = _make_cube()
     weights = get_time_weights(cube)
-    assert weights.shape == cube.shape
-    np.testing.assert_allclose(
-        weights, [[[[15.0, 15.0, 15.0]]], [[[30.0, 30.0, 30.0]]]])
+    assert weights.shape == (2, )
+    np.testing.assert_allclose(weights, [15.0, 30.0])
 
 
 def test_get_time_weights_0d_time():
@@ -1455,8 +1831,8 @@ def test_get_time_weights_0d_time():
                           units='K',
                           aux_coords_and_dims=[(time, ())])
     weights = get_time_weights(cube)
-    assert weights.shape == cube.shape
-    np.testing.assert_allclose(weights, 30.0)
+    assert weights.shape == (1, )
+    np.testing.assert_allclose(weights, [30.0])
 
 
 def test_get_time_weights_0d_time_1d_lon():
@@ -1469,8 +1845,8 @@ def test_get_time_weights_0d_time_1d_lon():
                           aux_coords_and_dims=[(time, ())],
                           dim_coords_and_dims=[(lons, 0)])
     weights = get_time_weights(cube)
-    assert weights.shape == cube.shape
-    np.testing.assert_allclose(weights, [30.0, 30.0, 30.0])
+    assert weights.shape == (1, )
+    np.testing.assert_allclose(weights, [30.0])
 
 
 def test_get_time_weights_1d_time():
@@ -1481,7 +1857,7 @@ def test_get_time_weights_1d_time():
                           units='K',
                           dim_coords_and_dims=[(time, 0)])
     weights = get_time_weights(cube)
-    assert weights.shape == cube.shape
+    assert weights.shape == (2, )
     np.testing.assert_allclose(weights, [15.0, 30.0])
 
 
@@ -1494,9 +1870,19 @@ def test_get_time_weights_1d_time_1d_lon():
                           units='K',
                           dim_coords_and_dims=[(time, 0), (lons, 1)])
     weights = get_time_weights(cube)
-    assert weights.shape == cube.shape
-    np.testing.assert_allclose(weights,
-                               [[15.0, 15.0, 15.0], [30.0, 30.0, 30.0]])
+    assert weights.shape == (2, )
+    np.testing.assert_allclose(weights, [15.0, 30.0])
+
+
+def test_get_time_weights_2d_time():
+    """Test ``get_time_weights`` for 1D time coordinate."""
+    time = get_2d_time()
+    cube = iris.cube.Cube([[0.0, 1.0]],
+                          var_name='x',
+                          units='K',
+                          aux_coords_and_dims=[(time, (0, 1))])
+    with pytest.raises(ValueError):
+        get_time_weights(cube)
 
 
 def test_climate_statistics_0d_time_1d_lon():
@@ -1517,7 +1903,7 @@ def test_climate_statistics_0d_time_1d_lon():
     np.testing.assert_allclose(new_cube.data, [1.0, -1.0, 42.0])
 
 
-def test_climate_statistics_complex_cube():
+def test_climate_statistics_complex_cube_sum():
     """Test climate statistics."""
     cube = _make_cube()
     new_cube = climate_statistics(cube, operator='sum', period='full')
@@ -1526,8 +1912,17 @@ def test_climate_statistics_complex_cube():
     np.testing.assert_allclose(new_cube.data, [[[45.0, 45.0, 45.0]]])
 
 
+def test_climate_statistics_complex_cube_mean():
+    """Test climate statistics."""
+    cube = _make_cube()
+    new_cube = climate_statistics(cube, operator='mean', period='full')
+    assert cube.shape == (2, 1, 1, 3)
+    assert new_cube.shape == (1, 1, 3)
+    np.testing.assert_allclose(new_cube.data, [[[1.0, 1.0, 1.0]]])
+
+
 class TestResampleHours(tests.Test):
-    """Test :func:`esmvalcore.preprocessor._time.resample_hours`"""
+    """Test :func:`esmvalcore.preprocessor._time.resample_hours`."""
     @staticmethod
     def _create_cube(data, times):
         time = iris.coords.DimCoord(times,
@@ -1614,9 +2009,18 @@ class TestResampleHours(tests.Test):
         with self.assertRaises(ValueError):
             resample_hours(cube, interval=12)
 
+    def test_resample_nodata(self):
+        """Test average of a 1D field."""
+        data = np.arange(0, 4, 1)
+        times = np.arange(0, 4, 1)
+        cube = self._create_cube(data, times)
+
+        with self.assertRaises(ValueError):
+            resample_hours(cube, offset=5, interval=6)
+
 
 class TestResampleTime(tests.Test):
-    """Test :func:`esmvalcore.preprocessor._time.resample_hours`"""
+    """Test :func:`esmvalcore.preprocessor._time.resample_time`."""
     @staticmethod
     def _create_cube(data, times):
         time = iris.coords.DimCoord(times,
@@ -1637,6 +2041,17 @@ class TestResampleTime(tests.Test):
         expected = np.arange(12, 48, 24)
         assert_array_equal(result.data, expected)
 
+    def test_scalar_cube(self):
+        """Test average of a 1D field."""
+        data = np.arange(0, 2, 1)
+        times = np.arange(0, 2, 1)
+        cube = self._create_cube(data, times)
+        cube = cube[0]
+
+        result = resample_time(cube, hour=0)
+        expected = np.zeros((1,))
+        assert_array_equal(result.data, expected)
+
     def test_resample_hourly_to_monthly(self):
         """Test average of a 1D field."""
         data = np.arange(0, 24 * 60, 3)
@@ -1660,6 +2075,25 @@ class TestResampleTime(tests.Test):
         ])
         assert_array_equal(result.data, expected)
 
+    def test_resample_fails(self):
+        """Test that selecting something that is not in the data fails."""
+        data = np.arange(0, 15 * 24, 24)
+        times = np.arange(0, 15 * 24, 24)
+        cube = self._create_cube(data, times)
+
+        with pytest.raises(ValueError):
+            resample_time(cube, day=16)
+
+    def test_resample_fails_scalar(self):
+        """Test that selecting something that is not in the data fails."""
+        data = np.arange(0, 2 * 24, 24)
+        times = np.arange(0, 2 * 24, 24)
+        cube = self._create_cube(data, times)
+        cube = cube[0]
+
+        with pytest.raises(ValueError):
+            resample_time(cube, day=16)
+
 
 if __name__ == '__main__':
     unittest.main()
